# Copyright (c) 2019 The Johns Hopkins University/Applied Physics Laboratory
# All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import testtools

from kmip.core import enums
from kmip.core import exceptions
from kmip.core import objects
from kmip.core import primitives
from kmip.core import utils


class TestCurrentAttribute(testtools.TestCase):
    """
    A unit test suite for the CurrentAttribute structure.
    """

    def setUp(self):
        super(TestCurrentAttribute, self).setUp()

        # This encoding matches the following set of values:
        # CurrentAttribute
        #     Cryptographic Algorithm - AES
        self.full_encoding = utils.BytearrayStream(
            b'\x42\x01\x3C\x01\x00\x00\x00\x10'
            b'\x42\x00\x28\x05\x00\x00\x00\x04\x00\x00\x00\x03\x00\x00\x00\x00'
        )

        self.empty_encoding = utils.BytearrayStream(
            b'\x42\x01\x3C\x01\x00\x00\x00\x00'
        )

        # This encoding matches the following set of values:
        # CurrentAttribute
        #     Non-existent Tag
        self.invalid_encoding = utils.BytearrayStream(
            b'\x42\x01\x3C\x01\x00\x00\x00\x10'
            b'\x42\xFF\xFF\x05\x00\x00\x00\x04\x00\x00\x00\x01\x00\x00\x00\x00'
        )

        # This encoding matches the following set of values:
        # CurrentAttribute
        #     Operation Policy Name - b4faee10-aa2a-4446-8ad4-0881f3422959
        self.unsupported_encoding = utils.BytearrayStream(
            b'\x42\x01\x3C\x01\x00\x00\x00\x30'
            b'\x42\x00\x5D\x07\x00\x00\x00\x24\x62\x34\x66\x61\x65\x65\x31\x30'
            b'\x2D\x61\x61\x32\x61\x2D\x34\x34\x34\x36\x2D\x38\x61\x64\x34\x2D'
            b'\x30\x38\x38\x31\x66\x33\x34\x32\x32\x39\x35\x39\x00\x00\x00\x00'
        )

    def tearDown(self):
        super(TestCurrentAttribute, self).tearDown()

    def test_unrecognized_attribute(self):
        """
        Test that a TypeError is raised when an unrecognized attribute is used
        to create a CurrentAttribute object. Note that this unrecognized
        attribute is a valid PyKMIP object derived from Base, it just isn't an
        attribute.
        """
        kwargs = {
            "attribute": primitives.Enumeration(
                enums.WrappingMethod,
                enums.WrappingMethod.ENCRYPT,
                enums.Tags.WRAPPING_METHOD
            )
        }
        self.assertRaisesRegex(
            TypeError,
            "The attribute must be a supported attribute type.",
            objects.CurrentAttribute,
            **kwargs
        )

        args = (
            objects.CurrentAttribute(),
            "attribute",
            primitives.Enumeration(
                enums.WrappingMethod,
                enums.WrappingMethod.ENCRYPT,
                enums.Tags.WRAPPING_METHOD
            )
        )
        self.assertRaisesRegex(
            TypeError,
            "The attribute must be a supported attribute type.",
            setattr,
            *args
        )

    def test_invalid_attribute(self):
        """
        Test that a TypeError is raised when an invalid value is used to
        create a CurrentAttribute object. Note that the value is not a valid
        PyKMIP object derived from Base and therefore cannot be an attribute.
        """
        kwargs = {"attribute": "invalid"}
        self.assertRaisesRegex(
            TypeError,
            "The attribute must be a Base object, not a {}.".format(
                type("invalid")
            ),
            objects.CurrentAttribute,
            **kwargs
        )

        args = (
            objects.CurrentAttribute(),
            "attribute",
            "invalid"
        )
        self.assertRaisesRegex(
            TypeError,
            "The attribute must be a Base object, not a {}.".format(
                type("invalid")
            ),
            setattr,
            *args
        )

    def test_read(self):
        """
        Test that a CurrentAttribute structure can be correctly read in from
        a data stream.
        """
        current_attribute = objects.CurrentAttribute()

        self.assertIsNone(current_attribute.attribute)

        current_attribute.read(self.full_encoding)

        self.assertIsInstance(
            current_attribute.attribute,
            primitives.Enumeration
        )
        self.assertEqual(
            current_attribute.attribute.value,
            enums.CryptographicAlgorithm.AES
        )

    def test_read_no_attribute(self):
        """
        Test that an InvalidKmipEncoding error is raised when an invalid
        encoding with no encoded attribute is used to decode a CurrentAttribute
        object.
        """
        current_attribute = objects.CurrentAttribute()
        args = (self.empty_encoding, )
        self.assertRaisesRegex(
            exceptions.InvalidKmipEncoding,
            "The CurrentAttribute encoding is missing the attribute field.",
            current_attribute.read,
            *args
        )

    def test_read_invalid_attribute(self):
        """
        Test that an InvalidKmipEncoding error is raised when an invalid
        encoding containing an invalid attribute is used to decode a
        CurrentAttribute object.
        """
        current_attribute = objects.CurrentAttribute()
        args = (self.invalid_encoding, )
        self.assertRaisesRegex(
            exceptions.InvalidKmipEncoding,
            "The CurrentAttribute encoding is missing the attribute field.",
            current_attribute.read,
            *args
        )

    def test_read_unsupported_attribute(self):
        """
        Test that an AttributeNotSupported error is raised when an unsupported
        attribute is parsed while reading in a CurrentAttribute object from a
        data stream. This can occur when an older attribute is no longer
        supported by a newer version of KMIP, or vice versa.
        """
        current_attribute = objects.CurrentAttribute()
        args = (self.unsupported_encoding, )
        kwargs = {"kmip_version": enums.KMIPVersion.KMIP_2_0}
        self.assertRaisesRegex(
            exceptions.AttributeNotSupported,
            "Attribute OPERATION_POLICY_NAME is not supported by KMIP 2.0.",
            current_attribute.read,
            *args,
            **kwargs
        )

    def test_read_version_not_supported(self):
        """
        Test that a VersionNotSupported error is raised when an unsupported
        KMIP version is provided while reading in a CurrentAttribute object
        from a data stream. The CurrentAttribute structure is only supported
        in KMIP 2.0+.
        """
        current_attribute = objects.CurrentAttribute()
        args = (self.full_encoding, )
        kwargs = {"kmip_version": enums.KMIPVersion.KMIP_1_2}
        self.assertRaisesRegex(
            exceptions.VersionNotSupported,
            "KMIP 1.2 does not support the CurrentAttribute object.",
            current_attribute.read,
            *args,
            **kwargs
        )

    def test_write(self):
        """
        Test that a CurrentAttribute object can be written to a data stream.
        """
        current_attribute = objects.CurrentAttribute(
            attribute=primitives.Enumeration(
                enums.CryptographicAlgorithm,
                enums.CryptographicAlgorithm.AES,
                enums.Tags.CRYPTOGRAPHIC_ALGORITHM
            )
        )

        stream = utils.BytearrayStream()
        current_attribute.write(stream)

        self.assertEqual(len(self.full_encoding), len(stream))
        self.assertEqual(str(self.full_encoding), str(stream))

    def test_write_no_attribute(self):
        """
        Test that an InvalidField error is raised when an empty
        CurrentAttribute object is written to a data stream.
        """
        current_attribute = objects.CurrentAttribute()
        args = (utils.BytearrayStream(), )
        self.assertRaisesRegex(
            exceptions.InvalidField,
            "The CurrentAttribute object is missing the attribute field.",
            current_attribute.write,
            *args
        )

    def test_write_unsupported_attribute(self):
        """
        Test that an AttributeNotSupported error is raised when an unsupported
        attribute is found while writing a CurrentAttribute object to a data
        stream. This can occur when an older attribute is no longer supported
        by a newer version of KMIP, or vice versa.
        """
        current_attribute = objects.CurrentAttribute(
            attribute=primitives.TextString(
                "default",
                tag=enums.Tags.OPERATION_POLICY_NAME
            )
        )
        args = (utils.BytearrayStream(), )
        kwargs = {"kmip_version": enums.KMIPVersion.KMIP_2_0}
        self.assertRaisesRegex(
            exceptions.AttributeNotSupported,
            "Attribute OPERATION_POLICY_NAME is not supported by KMIP 2.0.",
            current_attribute.write,
            *args,
            **kwargs
        )

    def test_write_version_not_supported(self):
        """
        Test that a VersionNotSupported error is raised when an unsupported
        KMIP version is provided while writing a CurrentAttribute object to
        a data stream. The CurrentAttribute structure is only supported in
        KMIP 2.0+.
        """
        current_attribute = objects.CurrentAttribute()
        args = (utils.BytearrayStream(), )
        kwargs = {"kmip_version": enums.KMIPVersion.KMIP_1_4}
        self.assertRaisesRegex(
            exceptions.VersionNotSupported,
            "KMIP 1.4 does not support the CurrentAttribute object.",
            current_attribute.write,
            *args,
            **kwargs
        )

    def test_repr(self):
        """
        Test that repr can be applied to a CurrentAttribute object.
        """
        current_attribute = objects.CurrentAttribute(
            attribute=primitives.Enumeration(
                enums.CryptographicAlgorithm,
                enums.CryptographicAlgorithm.AES,
                enums.Tags.CRYPTOGRAPHIC_ALGORITHM
            )
        )
        self.assertEqual(
            "CurrentAttribute("
            "attribute=Enumeration("
            "enum=CryptographicAlgorithm, "
            "value=CryptographicAlgorithm.AES, "
            "tag=Tags.CRYPTOGRAPHIC_ALGORITHM))",
            repr(current_attribute)
        )

    def test_str(self):
        """
        Test that str can be applied to a CurrentAttribute object.
        """
        current_attribute = objects.CurrentAttribute(
            attribute=primitives.Enumeration(
                enums.CryptographicAlgorithm,
                enums.CryptographicAlgorithm.AES,
                enums.Tags.CRYPTOGRAPHIC_ALGORITHM
            )
        )
        self.assertEqual(
            '{"attribute": Enumeration('
            'enum=CryptographicAlgorithm, '
            'value=CryptographicAlgorithm.AES, '
            'tag=Tags.CRYPTOGRAPHIC_ALGORITHM)}',
            str(current_attribute)
        )

    def test_comparison(self):
        """
        Test that the equality/inequality operators return True/False when
        comparing two CurrentAttribute objects with the same data.
        """
        a = objects.CurrentAttribute()
        b = objects.CurrentAttribute()

        self.assertTrue(a == b)
        self.assertTrue(b == a)
        self.assertFalse(a != b)
        self.assertFalse(b != a)

        a = objects.CurrentAttribute(
            attribute=primitives.Enumeration(
                enums.CryptographicAlgorithm,
                enums.CryptographicAlgorithm.AES,
                enums.Tags.CRYPTOGRAPHIC_ALGORITHM
            )
        )
        b = objects.CurrentAttribute(
            attribute=primitives.Enumeration(
                enums.CryptographicAlgorithm,
                enums.CryptographicAlgorithm.AES,
                enums.Tags.CRYPTOGRAPHIC_ALGORITHM
            )
        )

        self.assertTrue(a == b)
        self.assertTrue(b == a)
        self.assertFalse(a != b)
        self.assertFalse(b != a)

    def test_comparison_on_different_attributes(self):
        """
        Test that the equality/inequality operators return False/True when
        comparing two CurrentAttribute objects with different attributes.
        """
        a = objects.CurrentAttribute(
            attribute=primitives.Enumeration(
                enums.CryptographicAlgorithm,
                enums.CryptographicAlgorithm.AES,
                enums.Tags.CRYPTOGRAPHIC_ALGORITHM
            )
        )
        b = objects.CurrentAttribute(
            attribute=primitives.Integer(
                128,
                enums.Tags.CRYPTOGRAPHIC_LENGTH
            )
        )

        self.assertFalse(a == b)
        self.assertFalse(b == a)
        self.assertTrue(a != b)
        self.assertTrue(b != a)

    def test_comparison_on_type_mismatch(self):
        """
        Test that the equality/inequality operators return False/True when
        comparing two CurrentAttribute objects with different types.
        """
        a = objects.CurrentAttribute()
        b = "invalid"

        self.assertFalse(a == b)
        self.assertFalse(b == a)
        self.assertTrue(a != b)
        self.assertTrue(b != a)



# Copyright (c) 2015 The Johns Hopkins University/Applied Physics Laboratory
# All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import testtools

from kmip.core import exceptions
from kmip.core import primitives
from kmip.core import utils


class TestBigInteger(testtools.TestCase):

    def setUp(self):
        super(TestBigInteger, self).setUp()

        # Encodings and values taken from Sections 5.1, 13.3 and 18.2 of the
        # KMIP 1.1 testing documentation.
        self.value_positive = int(
            '74570697368583857894612671217453076717255131155396275504564761583'
            '15899148268876158582639566401239193216235126746176682996459367959'
            '36793366865165780165066709295778050045731105353780121783233185565'
            '36420486996200625818559496541368747791032257508332162004121562017'
            '72772159096834586599791505043949123930975157363117571140205992199'
            '59827555693853730430222361950476764952992840295849053634702315874'
            '87536235568284292445148693873502200712082861995083783995720224553'
            '38838078028390162249415071016709848797960500969432640102143437177'
            '60785867099769472998343832254180691121895373077720157164352949735'
            '8482684822484513735382434823977')
        self.value_negative = -1000

        self.encoding_zero = utils.BytearrayStream(
            b'\x42\x00\x00\x04\x00\x00\x00\x08\x00\x00\x00\x00\x00\x00\x00'
            b'\x00')
        self.encoding_positive = utils.BytearrayStream(
            b'\x42\x00\x00\x04\x00\x00\x01\x00\x3B\x12\x45\x5D\x53\xC1\x81\x65'
            b'\x16\xC5\x18\x49\x3F\x63\x98\xAA\xFA\x72\xB1\x7D\xFA\x89\x4D\xB8'
            b'\x88\xA7\xD4\x8C\x0A\x47\xF6\x25\x79\xA4\xE6\x44\xF8\x6D\xA7\x11'
            b'\xFE\xC8\x50\xCD\xD9\xDB\xBD\x17\xF6\x9A\x44\x3D\x2E\xC1\xDD\x60'
            b'\xD3\xC6\x18\xFA\x74\xCD\xE5\xFD\xAF\xAB\xD6\xBA\xA2\x6E\xB0\xA3'
            b'\xAD\xB4\xDE\xF6\x48\x0F\xB1\x21\x8C\xD3\xB0\x83\xE2\x52\xE8\x85'
            b'\xB6\xF0\x72\x9F\x98\xB2\x14\x4D\x2B\x72\x29\x3E\x1B\x11\xD7\x33'
            b'\x93\xBC\x41\xF7\x5B\x15\xEE\x3D\x75\x69\xB4\x99\x5E\xD1\xA1\x44'
            b'\x25\xDA\x43\x19\xB7\xB2\x6B\x0E\x8F\xEF\x17\xC3\x75\x42\xAE\x5C'
            b'\x6D\x58\x49\xF8\x72\x09\x56\x7F\x39\x25\xA4\x7B\x01\x6D\x56\x48'
            b'\x59\x71\x7B\xC5\x7F\xCB\x45\x22\xD0\xAA\x49\xCE\x81\x6E\x5B\xE7'
            b'\xB3\x08\x81\x93\x23\x6E\xC9\xEF\xFF\x14\x08\x58\x04\x5B\x73\xC5'
            b'\xD7\x9B\xAF\x38\xF7\xC6\x7F\x04\xC5\xDC\xF0\xE3\x80\x6A\xD9\x82'
            b'\xD1\x25\x90\x58\xC3\x47\x3E\x84\x71\x79\xA8\x78\xF2\xC6\xB3\xBD'
            b'\x96\x8F\xB9\x9E\xA4\x6E\x91\x85\x89\x2F\x36\x76\xE7\x89\x65\xC2'
            b'\xAE\xD4\x87\x7B\xA3\x91\x7D\xF0\x7C\x5E\x92\x74\x74\xF1\x9E\x76'
            b'\x4B\xA6\x1D\xC3\x8D\x63\xBF\x29')
        self.encoding_negative = utils.BytearrayStream(
            b'\x42\x00\x00\x04\x00\x00\x00\x08\xFF\xFF\xFF\xFF\xFF\xFF\xFC'
            b'\x18')
        self.encoding_bad_length = utils.BytearrayStream(
            b'\x42\x00\x00\x04\x00\x00\x01\x01\x3B\x12\x45\x5D\x53\xC1\x81\x65'
            b'\x16\xC5\x18\x49\x3F\x63\x98\xAA\xFA\x72\xB1\x7D\xFA\x89\x4D\xB8'
            b'\x88\xA7\xD4\x8C\x0A\x47\xF6\x25\x79\xA4\xE6\x44\xF8\x6D\xA7\x11'
            b'\xFE\xC8\x50\xCD\xD9\xDB\xBD\x17\xF6\x9A\x44\x3D\x2E\xC1\xDD\x60'
            b'\xD3\xC6\x18\xFA\x74\xCD\xE5\xFD\xAF\xAB\xD6\xBA\xA2\x6E\xB0\xA3'
            b'\xAD\xB4\xDE\xF6\x48\x0F\xB1\x21\x8C\xD3\xB0\x83\xE2\x52\xE8\x85'
            b'\xB6\xF0\x72\x9F\x98\xB2\x14\x4D\x2B\x72\x29\x3E\x1B\x11\xD7\x33'
            b'\x93\xBC\x41\xF7\x5B\x15\xEE\x3D\x75\x69\xB4\x99\x5E\xD1\xA1\x44'
            b'\x25\xDA\x43\x19\xB7\xB2\x6B\x0E\x8F\xEF\x17\xC3\x75\x42\xAE\x5C'
            b'\x6D\x58\x49\xF8\x72\x09\x56\x7F\x39\x25\xA4\x7B\x01\x6D\x56\x48'
            b'\x59\x71\x7B\xC5\x7F\xCB\x45\x22\xD0\xAA\x49\xCE\x81\x6E\x5B\xE7'
            b'\xB3\x08\x81\x93\x23\x6E\xC9\xEF\xFF\x14\x08\x58\x04\x5B\x73\xC5'
            b'\xD7\x9B\xAF\x38\xF7\xC6\x7F\x04\xC5\xDC\xF0\xE3\x80\x6A\xD9\x82'
            b'\xD1\x25\x90\x58\xC3\x47\x3E\x84\x71\x79\xA8\x78\xF2\xC6\xB3\xBD'
            b'\x96\x8F\xB9\x9E\xA4\x6E\x91\x85\x89\x2F\x36\x76\xE7\x89\x65\xC2'
            b'\xAE\xD4\x87\x7B\xA3\x91\x7D\xF0\x7C\x5E\x92\x74\x74\xF1\x9E\x76'
            b'\x4B\xA6\x1D\xC3\x8D\x63\xBF\x29')

    def tearDown(self):
        super(TestBigInteger, self).tearDown()

    def test_init(self):
        """
        Test that a BigInteger can be instantiated.
        """
        big_int = primitives.BigInteger(1)
        self.assertEqual(1, big_int.value)

    def test_init_unset(self):
        """
        Test that a BigInteger can be instantiated with no input.
        """
        big_int = primitives.BigInteger()
        self.assertEqual(0, big_int.value)

    def test_init_big_positive(self):
        """
        Test that a BigInteger can be instantiated with large positive input.
        """
        big_int = primitives.BigInteger(self.value_positive)
        self.assertEqual(self.value_positive, big_int.value)

    def test_init_negative(self):
        """
        Test that a BigInteger can be instantiated with negative input.
        """
        big_int = primitives.BigInteger(self.value_negative)
        self.assertEqual(self.value_negative, big_int.value)

    def test_validate_on_invalid(self):
        """
        Test that a TypeError is thrown on input of invalid type (e.g., str).
        """
        self.assertRaises(TypeError, primitives.BigInteger, 'invalid')

    def test_read_zero(self):
        """
        Test that a BigInteger representing the value 0 can be read from a
        byte stream.
        """
        big_int = primitives.BigInteger()
        big_int.read(self.encoding_zero)
        self.assertEqual(0, big_int.value)

    def test_read_positive(self):
        """
        Test that a BigInteger representing a big positive value can be read
        from a byte stream.
        """
        big_int = primitives.BigInteger()
        big_int.read(self.encoding_positive)
        self.assertEqual(self.value_positive, big_int.value)

    def test_read_negative(self):
        """
        Test that a BigInteger representing a negative value can be read from
        a byte stream.
        """
        big_int = primitives.BigInteger()
        big_int.read(self.encoding_negative)
        self.assertEqual(self.value_negative, big_int.value)

    def test_read_on_invalid_length(self):
        """
        Test that an InvalidPrimitiveLength exception is thrown when attempting
        to decode a BigInteger with an invalid length.
        """
        big_int = primitives.BigInteger()
        self.assertRaises(
            exceptions.InvalidPrimitiveLength, big_int.read,
            self.encoding_bad_length)

    def test_write_zero(self):
        """
        Test that a BigInteger representing the value 0 can be read written to
        a byte stream.
        """
        stream = utils.BytearrayStream()
        big_int = primitives.BigInteger()
        big_int.write(stream)
        self.assertEqual(self.encoding_zero, stream)

    def test_write_positive(self):
        """
        Test that a BigInteger representing a big positive value can be written
        to a byte stream.
        """
        stream = utils.BytearrayStream()
        big_int = primitives.BigInteger(self.value_positive)
        big_int.write(stream)
        self.assertEqual(self.encoding_positive, stream)

    def test_write_negative(self):
        """
        Test that a BigInteger representing a negative value can be written to
        a byte stream.
        """
        stream = utils.BytearrayStream()
        big_int = primitives.BigInteger(self.value_negative)
        big_int.write(stream)
        self.assertEqual(self.encoding_negative, stream)

    def test_repr(self):
        """
        Test that the representation of a BigInteger is formatted properly.
        """
        long_int = primitives.BigInteger()
        value = "value={0}".format(long_int.value)
        tag = "tag={0}".format(long_int.tag)
        self.assertEqual(
            "BigInteger({0}, {1})".format(value, tag), repr(long_int))

    def test_str(self):
        """
        Test that the string representation of a BigInteger is formatted
        properly.
        """
        self.assertEqual("0", str(primitives.BigInteger()))

    def test_equal_on_equal(self):
        """
        Test that the equality operator returns True when comparing two
        BigIntegers.
        """
        a = primitives.BigInteger(1)
        b = primitives.BigInteger(1)

        self.assertTrue(a == b)
        self.assertTrue(b == a)

    def test_equal_on_equal_and_empty(self):
        """
        Test that the equality operator returns True when comparing two
        BigIntegers.
        """
        a = primitives.BigInteger()
        b = primitives.BigInteger()

        self.assertTrue(a == b)
        self.assertTrue(b == a)

    def test_equal_on_not_equal(self):
        """
        Test that the equality operator returns False when comparing two
        BigIntegers with different values.
        """
        a = primitives.BigInteger(1)
        b = primitives.BigInteger(2)

        self.assertFalse(a == b)
        self.assertFalse(b == a)

    def test_equal_on_type_mismatch(self):
        """
        Test that the equality operator returns False when comparing a
        BigInteger to a non-BigInteger object.
        """
        a = primitives.BigInteger()
        b = 'invalid'

        self.assertFalse(a == b)
        self.assertFalse(b == a)

    def test_not_equal_on_equal(self):
        """
        Test that the inequality operator returns False when comparing
        two BigIntegers with the same values.
        """
        a = primitives.BigInteger(1)
        b = primitives.BigInteger(1)

        self.assertFalse(a != b)
        self.assertFalse(b != a)

    def test_not_equal_on_equal_and_empty(self):
        """
        Test that the inequality operator returns False when comparing
        two BigIntegers.
        """
        a = primitives.BigInteger()
        b = primitives.BigInteger()

        self.assertFalse(a != b)
        self.assertFalse(b != a)

    def test_not_equal_on_not_equal(self):
        """
        Test that the inequality operator returns True when comparing two
        BigIntegers with different values.
        """
        a = primitives.BigInteger(1)
        b = primitives.BigInteger(2)

        self.assertTrue(a != b)
        self.assertTrue(b != a)

    def test_not_equal_on_type_mismatch(self):
        """
        Test that the inequality operator returns True when comparing a
        BigInteger to a non-BigInteger object.
        """
        a = primitives.BigInteger()
        b = 'invalid'

        self.assertTrue(a != b)
        self.assertTrue(b != a)



# Copyright (c) 2018 The Johns Hopkins University/Applied Physics Laboratory
# All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import logging
import mock
import multiprocessing
import os
import shutil
import signal
import tempfile
import testtools

from kmip.core import enums
from kmip.services.server import monitor


class TestMonitorUtilities(testtools.TestCase):

    def setUp(self):
        super(TestMonitorUtilities, self).setUp()

        self.tmp_dir = tempfile.mkdtemp()
        self.addCleanup(shutil.rmtree, self.tmp_dir)

    def test_get_json_files(self):
        """
        Test that all files ending in .json can be collected from a directory.
        """
        with open(os.path.join(self.tmp_dir, "policy_1.json"), "w") as f:
            f.write('{"policy_1": {}}\n')
        with open(os.path.join(self.tmp_dir, "policy_2.json"), "w") as f:
            f.write('{"policy_2": {}}\n')
        with open(os.path.join(self.tmp_dir, "policy_3.txt"), "w") as f:
            f.write('{"policy_3": {}}\n')

        result = monitor.get_json_files(self.tmp_dir)

        self.assertIsInstance(result, list)
        self.assertEqual(2, len(result))
        self.assertIn(os.path.join(self.tmp_dir, "policy_1.json"), result)
        self.assertIn(os.path.join(self.tmp_dir, "policy_2.json"), result)


POLICY_1 = """
{
    "policy_A": {
        "groups": {
            "group_A": {
                "SYMMETRIC_KEY": {
                    "GET": "ALLOW_ALL",
                    "DESTROY": "ALLOW_ALL"
                }
            }
        }
    }
}
"""
POLICY_2 = """
{
    "policy_B": {
        "groups": {
            "group_B": {
                "SYMMETRIC_KEY": {
                    "GET": "ALLOW_ALL",
                    "LOCATE": "ALLOW_ALL",
                    "DESTROY": "ALLOW_ALL"
                }
            }
        }
    },
    "policy_C": {
        "groups": {
            "group_C": {
                "SYMMETRIC_KEY": {
                    "GET": "ALLOW_ALL",
                    "DESTROY": "DISALLOW_ALL"
                }
            }
        }
    }
}
"""
POLICY_3 = """
{
    "policy_B": {
        "groups": {
            "group_B": {
                "SYMMETRIC_KEY": {
                    "GET": "DISALLOW_ALL",
                    "LOCATE": "DISALLOW_ALL",
                    "DESTROY": "DISALLOW_ALL"
                }
            }
        }
    }
}
"""
POLICY_4 = """
{
    "default": {
        "groups": {
            "group_B": {
                "SYMMETRIC_KEY": {
                    "GET": "DISALLOW_ALL",
                    "LOCATE": "DISALLOW_ALL",
                    "DESTROY": "DISALLOW_ALL"
                }
            }
        }
    }
}
"""
POLICY_5 = """
{
    "policy_B": {
        "groups": {
            "group_B": {
                "SYMMETRIC_KEY": {
                    "GET": "ALLOW_ALL",
                    "LOCATE": "ALLOW_ALL",
                    "DESTROY": "ALLOW_ALL"
                }
            }
        }
    },
    "policy_D": {
        "groups": {
            "group_D": {
                "SYMMETRIC_KEY": {
                    "GET": "ALLOW_ALL",
                    "DESTROY": "DISALLOW_ALL"
                }
            }
        }
    }
}
"""
POLICY_6 = """
{
    "policy_A": {
        "groups": {
            "group_A": {
                "SYMMETRIC_KEY": {
                    "GET": "ALLOW_ALL",
                    "DESTROY": "ALLOW_ALL"
                }
            }
        }
    },
    "policy_E": {
        "groups": {
            "group_E": {
                "SYMMETRIC_KEY": {
                    "GET": "ALLOW_ALL",
                    "CHECK": "ALLOW_OWNER",
                    "DESTROY": "ALLOW_ALL"
                }
            }
        }
    }
}
"""
POLICY_7 = """
{
    "policy_D": {
        "groups": {
            "group_D": {
                "SYMMETRIC_KEY": {
                    "GET": "DISALLOW_ALL",
                    "LOCATE": "DISALLOW_ALL",
                    "DESTROY": "DISALLOW_ALL"
                }
            }
        }
    }
}
"""


def write_file(path, file_name, content):
    with open(os.path.join(path, file_name), "w") as f:
        f.write("{}\n".format(content))


def side_effects(effects):
    for effect in effects:
        if isinstance(effect, bool):
            yield effect
        else:
            effect()
            yield False


def build_write_effect(path, file_name, content):
    def side_effect():
        write_file(path, file_name, content)
    return side_effect


def build_delete_effect(path, file_name):
    def side_effect():
        os.remove(os.path.join(path, file_name))
    return side_effect


class TestPolicyDirectoryMonitor(testtools.TestCase):

    def setUp(self):
        super(TestPolicyDirectoryMonitor, self).setUp()

        self.tmp_dir = tempfile.mkdtemp()
        self.addCleanup(shutil.rmtree, self.tmp_dir)

    def test_init(self):
        """
        Test that the PolicyDirectoryMonitor can be instantiated without error.
        """
        m = monitor.PolicyDirectoryMonitor(
            self.tmp_dir,
            multiprocessing.Manager().dict()
        )

        self.assertIsInstance(
            m.halt_trigger,
            multiprocessing.synchronize.Event
        )
        self.assertEqual(self.tmp_dir, m.policy_directory)
        self.assertEqual({}, m.file_timestamps)
        self.assertEqual({}, m.policy_cache)
        self.assertEqual([], m.policy_files)
        self.assertEqual({}, m.policy_map)
        self.assertEqual([], m.policy_store.keys())
        self.assertEqual(['default', 'public'], m.reserved_policies)
        self.assertIsInstance(m.logger, logging.Logger)

    def test_signal_handler(self):
        """
        Test that the signal handler for SIGINT and SIGTERM correctly stops
        the monitor.
        """
        m = monitor.PolicyDirectoryMonitor(
            self.tmp_dir,
            multiprocessing.Manager().dict()
        )
        m.stop = mock.MagicMock()
        handler = signal.getsignal(signal.SIGINT)

        m.stop.assert_not_called()
        handler(None, None)
        m.stop.assert_called()

    def test_stop(self):
        """
        Test that the PolicyDirectoryMonitor processes stop calls correctly.
        """
        m = monitor.PolicyDirectoryMonitor(
            self.tmp_dir,
            multiprocessing.Manager().dict()
        )

        self.assertFalse(m.halt_trigger.is_set())

        m.stop()

        self.assertTrue(m.halt_trigger.is_set())

    def test_run(self):
        """
        Test that the PolicyDirectoryMonitor can load policy files and track
        them properly.
        """
        m = monitor.PolicyDirectoryMonitor(
            self.tmp_dir,
            multiprocessing.Manager().dict()
        )
        m.logger = mock.MagicMock(logging.Logger)
        m.halt_trigger = mock.MagicMock(multiprocessing.synchronize.Event)
        m.halt_trigger.is_set.side_effect = [False, True]

        write_file(self.tmp_dir, "policy_1.json", POLICY_1)
        write_file(self.tmp_dir, "policy_2.json", POLICY_2)

        self.assertEqual({}, m.file_timestamps)
        self.assertEqual({}, m.policy_cache)
        self.assertEqual([], m.policy_files)
        self.assertEqual({}, m.policy_map)
        self.assertEqual([], m.policy_store.keys())

        m.run()

        m.logger.info.assert_any_call(
            "Starting up the operation policy file monitor."
        )
        m.logger.info.assert_any_call(
            "Loading policies for file: {}".format(
                os.path.join(self.tmp_dir, "policy_1.json")
            )
        )
        m.logger.info.assert_any_call("Loading policy: policy_A")
        m.logger.info.assert_any_call(
            "Loading policies for file: {}".format(
                os.path.join(self.tmp_dir, "policy_2.json")
            )
        )
        m.logger.info.assert_any_call("Loading policy: policy_B")
        m.logger.info.assert_any_call("Loading policy: policy_C")
        m.logger.info.assert_any_call(
            "Stopping the operation policy file monitor."
        )

        self.assertEqual(2, len(m.policy_files))
        path = os.path.join(self.tmp_dir, "policy_1.json")
        self.assertEqual(
            os.path.getmtime(path),
            m.file_timestamps.get(path, None)
        )
        self.assertIn(path, m.policy_files)
        self.assertEqual(path, m.policy_map.get("policy_A", None))

        path = os.path.join(self.tmp_dir, "policy_2.json")
        self.assertEqual(
            os.path.getmtime(path),
            m.file_timestamps.get(path, None)
        )
        self.assertIn(path, m.policy_files)
        self.assertEqual(path, m.policy_map.get("policy_B", None))
        self.assertEqual(path, m.policy_map.get("policy_C", None))

        self.assertEqual(
            {
                "policy_A": [],
                "policy_B": [],
                "policy_C": []
            },
            m.policy_cache
        )

        self.assertEqual(3, len(m.policy_store.keys()))
        self.assertEqual(
            {
                "groups": {
                    "group_A": {
                        enums.ObjectType.SYMMETRIC_KEY: {
                            enums.Operation.GET: enums.Policy.ALLOW_ALL,
                            enums.Operation.DESTROY: enums.Policy.ALLOW_ALL
                        }
                    }
                }
            },
            m.policy_store.get("policy_A", None)
        )
        self.assertEqual(
            {
                "groups": {
                    "group_B": {
                        enums.ObjectType.SYMMETRIC_KEY: {
                            enums.Operation.GET: enums.Policy.ALLOW_ALL,
                            enums.Operation.LOCATE: enums.Policy.ALLOW_ALL,
                            enums.Operation.DESTROY: enums.Policy.ALLOW_ALL
                        }
                    }
                }
            },
            m.policy_store.get("policy_B", None)
        )
        self.assertEqual(
            {
                "groups": {
                    "group_C": {
                        enums.ObjectType.SYMMETRIC_KEY: {
                            enums.Operation.GET: enums.Policy.ALLOW_ALL,
                            enums.Operation.DESTROY: enums.Policy.DISALLOW_ALL
                        }
                    }
                }
            },
            m.policy_store.get("policy_C", None)
        )

    def test_run_with_policy_overloading(self):
        """
        Test that the PolicyDirectoryMonitor can load policy files and track
        them properly, even when one policy overloads another existing policy.
        """
        m = monitor.PolicyDirectoryMonitor(
            self.tmp_dir,
            multiprocessing.Manager().dict()
        )
        m.logger = mock.MagicMock(logging.Logger)
        m.halt_trigger = mock.MagicMock(multiprocessing.synchronize.Event)
        m.halt_trigger.is_set.side_effect = [False, True]

        write_file(self.tmp_dir, "policy_1.json", POLICY_1)
        write_file(self.tmp_dir, "policy_2.json", POLICY_2)
        write_file(self.tmp_dir, "policy_3.json", POLICY_3)

        self.assertEqual({}, m.file_timestamps)
        self.assertEqual({}, m.policy_cache)
        self.assertEqual([], m.policy_files)
        self.assertEqual({}, m.policy_map)
        self.assertEqual([], m.policy_store.keys())

        m.run()

        m.logger.info.assert_any_call(
            "Starting up the operation policy file monitor."
        )
        m.logger.info.assert_any_call(
            "Loading policies for file: {}".format(
                os.path.join(self.tmp_dir, "policy_1.json")
            )
        )
        m.logger.info.assert_any_call("Loading policy: policy_A")
        m.logger.info.assert_any_call(
            "Loading policies for file: {}".format(
                os.path.join(self.tmp_dir, "policy_2.json")
            )
        )
        m.logger.info.assert_any_call("Loading policy: policy_B")
        m.logger.info.assert_any_call("Loading policy: policy_C")
        m.logger.info.assert_any_call(
            "Loading policies for file: {}".format(
                os.path.join(self.tmp_dir, "policy_3.json")
            )
        )
        m.logger.info.assert_any_call("Loading policy: policy_B")
        m.logger.debug.assert_any_call(
            "Policy 'policy_B' overwrites an existing policy."
        )
        m.logger.info.assert_any_call(
            "Stopping the operation policy file monitor."
        )

        self.assertEqual(3, len(m.policy_files))
        path = os.path.join(self.tmp_dir, "policy_1.json")
        self.assertEqual(
            os.path.getmtime(path),
            m.file_timestamps.get(path, None)
        )
        self.assertIn(path, m.policy_files)
        self.assertEqual(path, m.policy_map.get("policy_A", None))

        path = os.path.join(self.tmp_dir, "policy_2.json")
        self.assertEqual(
            os.path.getmtime(path),
            m.file_timestamps.get(path, None)
        )
        self.assertIn(path, m.policy_files)
        self.assertEqual(path, m.policy_map.get("policy_C", None))

        path = os.path.join(self.tmp_dir, "policy_3.json")
        self.assertEqual(
            os.path.getmtime(path),
            m.file_timestamps.get(path, None)
        )
        self.assertIn(path, m.policy_files)
        self.assertEqual(path, m.policy_map.get("policy_B", None))

        cache = m.policy_cache.get("policy_A")
        self.assertEqual(0, len(cache))
        cache = m.policy_cache.get("policy_B")
        self.assertEqual(1, len(cache))
        self.assertEqual(
            os.path.join(self.tmp_dir, "policy_2.json"),
            cache[0][1]
        )
        self.assertEqual(
            {
                'groups': {
                    'group_B': {
                        enums.ObjectType.SYMMETRIC_KEY: {
                            enums.Operation.GET:
                                enums.Policy.ALLOW_ALL,
                            enums.Operation.LOCATE:
                                enums.Policy.ALLOW_ALL,
                            enums.Operation.DESTROY:
                                enums.Policy.ALLOW_ALL
                        }
                    }
                }
            },
            cache[0][2]
        )

        self.assertEqual(3, len(m.policy_store.keys()))
        self.assertEqual(
            {
                "groups": {
                    "group_A": {
                        enums.ObjectType.SYMMETRIC_KEY: {
                            enums.Operation.GET: enums.Policy.ALLOW_ALL,
                            enums.Operation.DESTROY: enums.Policy.ALLOW_ALL
                        }
                    }
                }
            },
            m.policy_store.get("policy_A", None)
        )
        self.assertEqual(
            {
                "groups": {
                    "group_B": {
                        enums.ObjectType.SYMMETRIC_KEY: {
                            enums.Operation.GET: enums.Policy.DISALLOW_ALL,
                            enums.Operation.LOCATE: enums.Policy.DISALLOW_ALL,
                            enums.Operation.DESTROY: enums.Policy.DISALLOW_ALL
                        }
                    }
                }
            },
            m.policy_store.get("policy_B", None)
        )
        self.assertEqual(
            {
                "groups": {
                    "group_C": {
                        enums.ObjectType.SYMMETRIC_KEY: {
                            enums.Operation.GET: enums.Policy.ALLOW_ALL,
                            enums.Operation.DESTROY: enums.Policy.DISALLOW_ALL
                        }
                    }
                }
            },
            m.policy_store.get("policy_C", None)
        )

    def test_run_with_policy_load_failure(self):
        """
        Test that the PolicyDirectoryMonitor can load policy files and track
        them properly, even when one policy can't be loaded properly.
        """
        m = monitor.PolicyDirectoryMonitor(
            self.tmp_dir,
            multiprocessing.Manager().dict()
        )
        m.logger = mock.MagicMock(logging.Logger)
        m.halt_trigger = mock.MagicMock(multiprocessing.synchronize.Event)
        m.halt_trigger.is_set.side_effect = [False, True]

        write_file(self.tmp_dir, "policy_1.json", POLICY_1)
        write_file(self.tmp_dir, "policy_2.json", "not a JSON blob")

        self.assertEqual({}, m.file_timestamps)
        self.assertEqual({}, m.policy_cache)
        self.assertEqual([], m.policy_files)
        self.assertEqual({}, m.policy_map)
        self.assertEqual([], m.policy_store.keys())

        m.run()

        m.logger.info.assert_any_call(
            "Starting up the operation policy file monitor."
        )
        m.logger.info.assert_any_call(
            "Loading policies for file: {}".format(
                os.path.join(self.tmp_dir, "policy_1.json")
            )
        )
        m.logger.info.assert_any_call("Loading policy: policy_A")
        m.logger.info.assert_any_call(
            "Loading policies for file: {}".format(
                os.path.join(self.tmp_dir, "policy_2.json")
            )
        )
        m.logger.error.assert_any_call(
            "Failure loading file: {}".format(
                os.path.join(self.tmp_dir, "policy_2.json")
            )
        )
        m.logger.debug.assert_called()
        m.logger.info.assert_any_call(
            "Stopping the operation policy file monitor."
        )

        self.assertEqual(2, len(m.policy_files))
        path = os.path.join(self.tmp_dir, "policy_1.json")
        self.assertEqual(
            os.path.getmtime(path),
            m.file_timestamps.get(path, None)
        )
        self.assertIn(path, m.policy_files)
        self.assertEqual(path, m.policy_map.get("policy_A", None))

        path = os.path.join(self.tmp_dir, "policy_2.json")
        self.assertEqual(
            os.path.getmtime(path),
            m.file_timestamps.get(path, None)
        )
        self.assertIn(path, m.policy_files)

        self.assertEqual(
            {
                "policy_A": []
            },
            m.policy_cache
        )

        self.assertEqual(1, len(m.policy_store.keys()))
        self.assertEqual(
            {
                "groups": {
                    "group_A": {
                        enums.ObjectType.SYMMETRIC_KEY: {
                            enums.Operation.GET: enums.Policy.ALLOW_ALL,
                            enums.Operation.DESTROY: enums.Policy.ALLOW_ALL
                        }
                    }
                }
            },
            m.policy_store.get("policy_A", None)
        )

    def test_run_with_policy_load_failure_and_fix(self):
        """
        Test that the PolicyDirectoryMonitor can load policy files and track
        them properly, even when one policy can't be loaded properly and is
        then fixed while tracking is active.
        """
        m = monitor.PolicyDirectoryMonitor(
            self.tmp_dir,
            multiprocessing.Manager().dict()
        )
        m.logger = mock.MagicMock(logging.Logger)
        m.halt_trigger = mock.MagicMock(multiprocessing.synchronize.Event)
        m.halt_trigger.is_set.side_effect = side_effects(
            [
                False,
                build_write_effect(
                    self.tmp_dir,
                    "policy_2.json",
                    "invalid JSON"
                ),
                False,
                build_write_effect(self.tmp_dir, "policy_2.json", POLICY_2),
                False,
                True
            ]
        )

        write_file(self.tmp_dir, "policy_1.json", POLICY_1)

        self.assertEqual({}, m.file_timestamps)
        self.assertEqual({}, m.policy_cache)
        self.assertEqual([], m.policy_files)
        self.assertEqual({}, m.policy_map)
        self.assertEqual([], m.policy_store.keys())

        m.run()

        m.logger.info.assert_any_call(
            "Starting up the operation policy file monitor."
        )
        m.logger.info.assert_any_call(
            "Loading policies for file: {}".format(
                os.path.join(self.tmp_dir, "policy_1.json")
            )
        )
        m.logger.info.assert_any_call("Loading policy: policy_A")
        m.logger.info.assert_any_call(
            "Loading policies for file: {}".format(
                os.path.join(self.tmp_dir, "policy_2.json")
            )
        )
        m.logger.error.assert_any_call(
            "Failure loading file: {}".format(
                os.path.join(self.tmp_dir, "policy_2.json")
            )
        )
        m.logger.debug.assert_called()
        m.logger.info.assert_any_call(
            "Loading policies for file: {}".format(
                os.path.join(self.tmp_dir, "policy_2.json")
            )
        )
        m.logger.info.assert_any_call("Loading policy: policy_B")
        m.logger.info.assert_any_call("Loading policy: policy_C")
        m.logger.info.assert_any_call(
            "Stopping the operation policy file monitor."
        )

        self.assertEqual(2, len(m.policy_files))
        path = os.path.join(self.tmp_dir, "policy_1.json")
        self.assertEqual(
            os.path.getmtime(path),
            m.file_timestamps.get(path, None)
        )
        self.assertIn(path, m.policy_files)
        self.assertEqual(path, m.policy_map.get("policy_A", None))

        path = os.path.join(self.tmp_dir, "policy_2.json")
        self.assertEqual(
            os.path.getmtime(path),
            m.file_timestamps.get(path, None)
        )
        self.assertIn(path, m.policy_files)
        self.assertEqual(path, m.policy_map.get("policy_B", None))
        self.assertEqual(path, m.policy_map.get("policy_C", None))

        self.assertEqual(
            {
                "policy_A": [],
                "policy_B": [],
                "policy_C": []
            },
            m.policy_cache
        )

        self.assertEqual(3, len(m.policy_store.keys()))
        self.assertEqual(
            {
                "groups": {
                    "group_A": {
                        enums.ObjectType.SYMMETRIC_KEY: {
                            enums.Operation.GET: enums.Policy.ALLOW_ALL,
                            enums.Operation.DESTROY: enums.Policy.ALLOW_ALL
                        }
                    }
                }
            },
            m.policy_store.get("policy_A", None)
        )
        self.assertEqual(
            {
                "groups": {
                    "group_B": {
                        enums.ObjectType.SYMMETRIC_KEY: {
                            enums.Operation.GET: enums.Policy.ALLOW_ALL,
                            enums.Operation.LOCATE: enums.Policy.ALLOW_ALL,
                            enums.Operation.DESTROY: enums.Policy.ALLOW_ALL
                        }
                    }
                }
            },
            m.policy_store.get("policy_B", None)
        )
        self.assertEqual(
            {
                "groups": {
                    "group_C": {
                        enums.ObjectType.SYMMETRIC_KEY: {
                            enums.Operation.GET: enums.Policy.ALLOW_ALL,
                            enums.Operation.DESTROY: enums.Policy.DISALLOW_ALL
                        }
                    }
                }
            },
            m.policy_store.get("policy_C", None)
        )

    def test_run_with_policy_overloading_reserved(self):
        """
        Test that the PolicyDirectoryMonitor can load policy files and track
        them properly, even when one policy can't be loaded properly.
        """
        m = monitor.PolicyDirectoryMonitor(
            self.tmp_dir,
            multiprocessing.Manager().dict()
        )
        m.logger = mock.MagicMock(logging.Logger)
        m.halt_trigger = mock.MagicMock(multiprocessing.synchronize.Event)
        m.halt_trigger.is_set.side_effect = [False, True]

        write_file(self.tmp_dir, "policy_3.json", POLICY_3)
        write_file(self.tmp_dir, "policy_4.json", POLICY_4)

        self.assertEqual({}, m.file_timestamps)
        self.assertEqual({}, m.policy_cache)
        self.assertEqual([], m.policy_files)
        self.assertEqual({}, m.policy_map)
        self.assertEqual([], m.policy_store.keys())

        m.run()

        m.logger.info.assert_any_call(
            "Starting up the operation policy file monitor."
        )
        m.logger.info.assert_any_call(
            "Loading policies for file: {}".format(
                os.path.join(self.tmp_dir, "policy_3.json")
            )
        )
        m.logger.info.assert_any_call("Loading policy: policy_B")
        m.logger.info.assert_any_call(
            "Loading policies for file: {}".format(
                os.path.join(self.tmp_dir, "policy_4.json")
            )
        )
        m.logger.info.assert_any_call("Loading policy: default")
        m.logger.warning.assert_any_call(
            "Policy 'default' overwrites a reserved policy and will be "
            "thrown out."
        )
        m.logger.info.assert_any_call(
            "Stopping the operation policy file monitor."
        )

        self.assertEqual(2, len(m.policy_files))
        path = os.path.join(self.tmp_dir, "policy_3.json")
        self.assertEqual(
            os.path.getmtime(path),
            m.file_timestamps.get(path, None)
        )
        self.assertIn(path, m.policy_files)
        self.assertEqual(path, m.policy_map.get("policy_B", None))

        path = os.path.join(self.tmp_dir, "policy_4.json")
        self.assertEqual(
            os.path.getmtime(path),
            m.file_timestamps.get(path, None)
        )
        self.assertIn(path, m.policy_files)

        self.assertEqual(
            {
                "policy_B": []
            },
            m.policy_cache
        )

        self.assertEqual(1, len(m.policy_store.keys()))
        self.assertEqual(
            {
                "groups": {
                    "group_B": {
                        enums.ObjectType.SYMMETRIC_KEY: {
                            enums.Operation.GET: enums.Policy.DISALLOW_ALL,
                            enums.Operation.LOCATE: enums.Policy.DISALLOW_ALL,
                            enums.Operation.DESTROY: enums.Policy.DISALLOW_ALL
                        }
                    }
                }
            },
            m.policy_store.get("policy_B", None)
        )

    def test_run_with_edit_modifying_existing_file(self):
        """
        Test that the PolicyDirectoryMonitor can load policy files and track
        them properly, even when an existing policy file is modified while
        tracking is active.
        """
        m = monitor.PolicyDirectoryMonitor(
            self.tmp_dir,
            multiprocessing.Manager().dict()
        )
        m.logger = mock.MagicMock(logging.Logger)
        m.halt_trigger = mock.MagicMock(multiprocessing.synchronize.Event)
        m.halt_trigger.is_set.side_effect = side_effects(
            [
                False,
                build_write_effect(self.tmp_dir, "policy_2.json", POLICY_5),
                True
            ]
        )

        write_file(self.tmp_dir, "policy_1.json", POLICY_1)
        write_file(self.tmp_dir, "policy_2.json", POLICY_2)

        self.assertEqual({}, m.file_timestamps)
        self.assertEqual({}, m.policy_cache)
        self.assertEqual([], m.policy_files)
        self.assertEqual({}, m.policy_map)
        self.assertEqual([], m.policy_store.keys())

        m.run()

        m.logger.info.assert_any_call(
            "Starting up the operation policy file monitor."
        )
        m.logger.info.assert_any_call(
            "Loading policies for file: {}".format(
                os.path.join(self.tmp_dir, "policy_1.json")
            )
        )
        m.logger.info.assert_any_call("Loading policy: policy_A")
        m.logger.info.assert_any_call(
            "Loading policies for file: {}".format(
                os.path.join(self.tmp_dir, "policy_2.json")
            )
        )
        m.logger.info.assert_any_call("Loading policy: policy_B")
        m.logger.info.assert_any_call("Loading policy: policy_C")
        m.logger.info.assert_any_call(
            "Loading policies for file: {}".format(
                os.path.join(self.tmp_dir, "policy_2.json")
            )
        )
        m.logger.info.assert_any_call("Loading policy: policy_B")
        m.logger.info.assert_any_call("Loading policy: policy_D")
        m.logger.info.assert_any_call("Removing policy: policy_C")
        m.logger.info.assert_any_call(
            "Stopping the operation policy file monitor."
        )

        self.assertEqual(2, len(m.policy_files))
        path = os.path.join(self.tmp_dir, "policy_1.json")
        self.assertEqual(
            os.path.getmtime(path),
            m.file_timestamps.get(path, None)
        )
        self.assertIn(path, m.policy_files)
        self.assertEqual(path, m.policy_map.get("policy_A", None))

        path = os.path.join(self.tmp_dir, "policy_2.json")
        self.assertEqual(
            os.path.getmtime(path),
            m.file_timestamps.get(path, None)
        )
        self.assertIn(path, m.policy_files)
        self.assertEqual(path, m.policy_map.get("policy_B", None))
        self.assertEqual(path, m.policy_map.get("policy_D", None))

        self.assertEqual(
            {
                "policy_A": [],
                "policy_B": [],
                "policy_D": []
            },
            m.policy_cache
        )

        self.assertEqual(3, len(m.policy_store.keys()))
        self.assertEqual(
            {
                "groups": {
                    "group_A": {
                        enums.ObjectType.SYMMETRIC_KEY: {
                            enums.Operation.GET: enums.Policy.ALLOW_ALL,
                            enums.Operation.DESTROY: enums.Policy.ALLOW_ALL
                        }
                    }
                }
            },
            m.policy_store.get("policy_A", None)
        )
        self.assertEqual(
            {
                "groups": {
                    "group_B": {
                        enums.ObjectType.SYMMETRIC_KEY: {
                            enums.Operation.GET: enums.Policy.ALLOW_ALL,
                            enums.Operation.LOCATE: enums.Policy.ALLOW_ALL,
                            enums.Operation.DESTROY: enums.Policy.ALLOW_ALL
                        }
                    }
                }
            },
            m.policy_store.get("policy_B", None)
        )
        self.assertEqual(
            {
                "groups": {
                    "group_D": {
                        enums.ObjectType.SYMMETRIC_KEY: {
                            enums.Operation.GET: enums.Policy.ALLOW_ALL,
                            enums.Operation.DESTROY: enums.Policy.DISALLOW_ALL
                        }
                    }
                }
            },
            m.policy_store.get("policy_D", None)
        )

    def test_run_with_edit_adding_to_existing_file(self):
        """
        Test that the PolicyDirectoryMonitor can load policy files and track
        them properly, even when an existing policy file is added to while
        tracking is active.
        """
        m = monitor.PolicyDirectoryMonitor(
            self.tmp_dir,
            multiprocessing.Manager().dict()
        )
        m.logger = mock.MagicMock(logging.Logger)
        m.halt_trigger = mock.MagicMock(multiprocessing.synchronize.Event)
        m.halt_trigger.is_set.side_effect = side_effects(
            [
                False,
                build_write_effect(self.tmp_dir, "policy_1.json", POLICY_6),
                True
            ]
        )

        write_file(self.tmp_dir, "policy_1.json", POLICY_1)
        write_file(self.tmp_dir, "policy_2.json", POLICY_2)

        self.assertEqual({}, m.file_timestamps)
        self.assertEqual({}, m.policy_cache)
        self.assertEqual([], m.policy_files)
        self.assertEqual({}, m.policy_map)
        self.assertEqual([], m.policy_store.keys())

        m.run()

        m.logger.info.assert_any_call(
            "Starting up the operation policy file monitor."
        )
        m.logger.info.assert_any_call(
            "Loading policies for file: {}".format(
                os.path.join(self.tmp_dir, "policy_1.json")
            )
        )
        m.logger.info.assert_any_call("Loading policy: policy_A")
        m.logger.info.assert_any_call(
            "Loading policies for file: {}".format(
                os.path.join(self.tmp_dir, "policy_2.json")
            )
        )
        m.logger.info.assert_any_call("Loading policy: policy_B")
        m.logger.info.assert_any_call("Loading policy: policy_C")
        m.logger.info.assert_any_call(
            "Loading policies for file: {}".format(
                os.path.join(self.tmp_dir, "policy_1.json")
            )
        )
        m.logger.info.assert_any_call("Loading policy: policy_A")
        m.logger.info.assert_any_call("Loading policy: policy_E")
        m.logger.info.assert_any_call(
            "Stopping the operation policy file monitor."
        )

        self.assertEqual(2, len(m.policy_files))
        path = os.path.join(self.tmp_dir, "policy_1.json")
        self.assertEqual(
            os.path.getmtime(path),
            m.file_timestamps.get(path, None)
        )
        self.assertIn(path, m.policy_files)
        self.assertEqual(path, m.policy_map.get("policy_A", None))
        self.assertEqual(path, m.policy_map.get("policy_E", None))

        path = os.path.join(self.tmp_dir, "policy_2.json")
        self.assertEqual(
            os.path.getmtime(path),
            m.file_timestamps.get(path, None)
        )
        self.assertIn(path, m.policy_files)
        self.assertEqual(path, m.policy_map.get("policy_B", None))
        self.assertEqual(path, m.policy_map.get("policy_C", None))

        self.assertEqual(
            {
                "policy_A": [],
                "policy_B": [],
                "policy_C": [],
                "policy_E": []
            },
            m.policy_cache
        )

        self.assertEqual(4, len(m.policy_store.keys()))
        self.assertEqual(
            {
                "groups": {
                    "group_A": {
                        enums.ObjectType.SYMMETRIC_KEY: {
                            enums.Operation.GET: enums.Policy.ALLOW_ALL,
                            enums.Operation.DESTROY: enums.Policy.ALLOW_ALL
                        }
                    }
                }
            },
            m.policy_store.get("policy_A", None)
        )
        self.assertEqual(
            {
                "groups": {
                    "group_B": {
                        enums.ObjectType.SYMMETRIC_KEY: {
                            enums.Operation.GET: enums.Policy.ALLOW_ALL,
                            enums.Operation.LOCATE: enums.Policy.ALLOW_ALL,
                            enums.Operation.DESTROY: enums.Policy.ALLOW_ALL
                        }
                    }
                }
            },
            m.policy_store.get("policy_B", None)
        )
        self.assertEqual(
            {
                "groups": {
                    "group_C": {
                        enums.ObjectType.SYMMETRIC_KEY: {
                            enums.Operation.GET: enums.Policy.ALLOW_ALL,
                            enums.Operation.DESTROY: enums.Policy.DISALLOW_ALL
                        }
                    }
                }
            },
            m.policy_store.get("policy_C", None)
        )
        self.assertEqual(
            {
                "groups": {
                    "group_E": {
                        enums.ObjectType.SYMMETRIC_KEY: {
                            enums.Operation.GET: enums.Policy.ALLOW_ALL,
                            enums.Operation.CHECK: enums.Policy.ALLOW_OWNER,
                            enums.Operation.DESTROY: enums.Policy.ALLOW_ALL
                        }
                    }
                }
            },
            m.policy_store.get("policy_E", None)
        )

    def test_run_with_edit_deleting_from_existing_file(self):
        """
        Test that the PolicyDirectoryMonitor can load policy files and track
        them properly, even when an existing policy file has content removed
        while tracking is active.
        """
        m = monitor.PolicyDirectoryMonitor(
            self.tmp_dir,
            multiprocessing.Manager().dict()
        )
        m.logger = mock.MagicMock(logging.Logger)
        m.halt_trigger = mock.MagicMock(multiprocessing.synchronize.Event)
        m.halt_trigger.is_set.side_effect = side_effects(
            [
                False,
                build_write_effect(self.tmp_dir, "policy_1.json", POLICY_1),
                True
            ]
        )

        write_file(self.tmp_dir, "policy_1.json", POLICY_6)
        write_file(self.tmp_dir, "policy_2.json", POLICY_2)

        self.assertEqual({}, m.file_timestamps)
        self.assertEqual({}, m.policy_cache)
        self.assertEqual([], m.policy_files)
        self.assertEqual({}, m.policy_map)
        self.assertEqual([], m.policy_store.keys())

        m.run()

        m.logger.info.assert_any_call(
            "Starting up the operation policy file monitor."
        )
        m.logger.info.assert_any_call(
            "Loading policies for file: {}".format(
                os.path.join(self.tmp_dir, "policy_1.json")
            )
        )
        m.logger.info.assert_any_call("Loading policy: policy_A")
        m.logger.info.assert_any_call("Loading policy: policy_E")
        m.logger.info.assert_any_call(
            "Loading policies for file: {}".format(
                os.path.join(self.tmp_dir, "policy_2.json")
            )
        )
        m.logger.info.assert_any_call("Loading policy: policy_B")
        m.logger.info.assert_any_call("Loading policy: policy_C")
        m.logger.info.assert_any_call(
            "Loading policies for file: {}".format(
                os.path.join(self.tmp_dir, "policy_1.json")
            )
        )
        m.logger.info.assert_any_call("Loading policy: policy_A")
        m.logger.info.assert_any_call("Removing policy: policy_E")
        m.logger.info.assert_any_call(
            "Stopping the operation policy file monitor."
        )

        self.assertEqual(2, len(m.policy_files))
        path = os.path.join(self.tmp_dir, "policy_1.json")
        self.assertEqual(
            os.path.getmtime(path),
            m.file_timestamps.get(path, None)
        )
        self.assertIn(path, m.policy_files)
        self.assertEqual(path, m.policy_map.get("policy_A", None))

        path = os.path.join(self.tmp_dir, "policy_2.json")
        self.assertEqual(
            os.path.getmtime(path),
            m.file_timestamps.get(path, None)
        )
        self.assertIn(path, m.policy_files)
        self.assertEqual(path, m.policy_map.get("policy_B", None))
        self.assertEqual(path, m.policy_map.get("policy_C", None))

        self.assertEqual(
            {
                "policy_A": [],
                "policy_B": [],
                "policy_C": []
            },
            m.policy_cache
        )

        self.assertEqual(3, len(m.policy_store.keys()))
        self.assertEqual(
            {
                "groups": {
                    "group_A": {
                        enums.ObjectType.SYMMETRIC_KEY: {
                            enums.Operation.GET: enums.Policy.ALLOW_ALL,
                            enums.Operation.DESTROY: enums.Policy.ALLOW_ALL
                        }
                    }
                }
            },
            m.policy_store.get("policy_A", None)
        )
        self.assertEqual(
            {
                "groups": {
                    "group_B": {
                        enums.ObjectType.SYMMETRIC_KEY: {
                            enums.Operation.GET: enums.Policy.ALLOW_ALL,
                            enums.Operation.LOCATE: enums.Policy.ALLOW_ALL,
                            enums.Operation.DESTROY: enums.Policy.ALLOW_ALL
                        }
                    }
                }
            },
            m.policy_store.get("policy_B", None)
        )
        self.assertEqual(
            {
                "groups": {
                    "group_C": {
                        enums.ObjectType.SYMMETRIC_KEY: {
                            enums.Operation.GET: enums.Policy.ALLOW_ALL,
                            enums.Operation.DESTROY: enums.Policy.DISALLOW_ALL
                        }
                    }
                }
            },
            m.policy_store.get("policy_C", None)
        )

    def test_run_with_deleting_existing_file(self):
        """
        Test that the PolicyDirectoryMonitor can load policy files and track
        them properly, even when an existing policy file is removed while
        tracking is active.
        """
        m = monitor.PolicyDirectoryMonitor(
            self.tmp_dir,
            multiprocessing.Manager().dict()
        )
        m.logger = mock.MagicMock(logging.Logger)
        m.halt_trigger = mock.MagicMock(multiprocessing.synchronize.Event)
        m.halt_trigger.is_set.side_effect = side_effects(
            [
                False,
                build_delete_effect(self.tmp_dir, "policy_1.json"),
                True
            ]
        )

        write_file(self.tmp_dir, "policy_1.json", POLICY_6)
        write_file(self.tmp_dir, "policy_2.json", POLICY_2)

        self.assertEqual({}, m.file_timestamps)
        self.assertEqual({}, m.policy_cache)
        self.assertEqual([], m.policy_files)
        self.assertEqual({}, m.policy_map)
        self.assertEqual([], m.policy_store.keys())

        m.run()

        m.logger.info.assert_any_call(
            "Starting up the operation policy file monitor."
        )
        m.logger.info.assert_any_call(
            "Loading policies for file: {}".format(
                os.path.join(self.tmp_dir, "policy_1.json")
            )
        )
        m.logger.info.assert_any_call("Loading policy: policy_A")
        m.logger.info.assert_any_call("Loading policy: policy_E")
        m.logger.info.assert_any_call(
            "Loading policies for file: {}".format(
                os.path.join(self.tmp_dir, "policy_2.json")
            )
        )
        m.logger.info.assert_any_call("Loading policy: policy_B")
        m.logger.info.assert_any_call("Loading policy: policy_C")
        m.logger.info.assert_any_call(
            "Removing policies for file: {}".format(
                os.path.join(self.tmp_dir, "policy_1.json")
            )
        )
        m.logger.info.assert_any_call("Removing policy: policy_A")
        m.logger.info.assert_any_call("Removing policy: policy_E")
        m.logger.info.assert_any_call(
            "Stopping the operation policy file monitor."
        )

        self.assertEqual(1, len(m.policy_files))
        path = os.path.join(self.tmp_dir, "policy_2.json")
        self.assertEqual(
            os.path.getmtime(path),
            m.file_timestamps.get(path, None)
        )
        self.assertIn(path, m.policy_files)
        self.assertEqual(2, len(m.policy_map.keys()))
        self.assertEqual(path, m.policy_map.get("policy_B", None))
        self.assertEqual(path, m.policy_map.get("policy_C", None))

        self.assertEqual(
            {
                "policy_B": [],
                "policy_C": []
            },
            m.policy_cache
        )

        self.assertEqual(2, len(m.policy_store.keys()))
        self.assertEqual(
            {
                "groups": {
                    "group_B": {
                        enums.ObjectType.SYMMETRIC_KEY: {
                            enums.Operation.GET: enums.Policy.ALLOW_ALL,
                            enums.Operation.LOCATE: enums.Policy.ALLOW_ALL,
                            enums.Operation.DESTROY: enums.Policy.ALLOW_ALL
                        }
                    }
                }
            },
            m.policy_store.get("policy_B", None)
        )
        self.assertEqual(
            {
                "groups": {
                    "group_C": {
                        enums.ObjectType.SYMMETRIC_KEY: {
                            enums.Operation.GET: enums.Policy.ALLOW_ALL,
                            enums.Operation.DESTROY: enums.Policy.DISALLOW_ALL
                        }
                    }
                }
            },
            m.policy_store.get("policy_C", None)
        )

    def test_run_with_adding_new_file(self):
        """
        Test that the PolicyDirectoryMonitor can load policy files and track
        them properly, even when a new policy file is added while tracking is
        active.
        """
        m = monitor.PolicyDirectoryMonitor(
            self.tmp_dir,
            multiprocessing.Manager().dict()
        )
        m.logger = mock.MagicMock(logging.Logger)
        m.halt_trigger = mock.MagicMock(multiprocessing.synchronize.Event)
        m.halt_trigger.is_set.side_effect = side_effects(
            [
                False,
                build_write_effect(self.tmp_dir, "policy_2.json", POLICY_2),
                True
            ]
        )

        write_file(self.tmp_dir, "policy_1.json", POLICY_1)

        self.assertEqual({}, m.file_timestamps)
        self.assertEqual({}, m.policy_cache)
        self.assertEqual([], m.policy_files)
        self.assertEqual({}, m.policy_map)
        self.assertEqual([], m.policy_store.keys())

        m.run()

        m.logger.info.assert_any_call(
            "Starting up the operation policy file monitor."
        )
        m.logger.info.assert_any_call(
            "Loading policies for file: {}".format(
                os.path.join(self.tmp_dir, "policy_1.json")
            )
        )
        m.logger.info.assert_any_call("Loading policy: policy_A")
        m.logger.info.assert_any_call(
            "Loading policies for file: {}".format(
                os.path.join(self.tmp_dir, "policy_2.json")
            )
        )
        m.logger.debug.assert_not_called()
        m.logger.info.assert_any_call("Loading policy: policy_B")
        m.logger.info.assert_any_call("Loading policy: policy_C")
        m.logger.info.assert_any_call(
            "Stopping the operation policy file monitor."
        )

        self.assertEqual(2, len(m.policy_files))
        path = os.path.join(self.tmp_dir, "policy_1.json")
        self.assertEqual(
            os.path.getmtime(path),
            m.file_timestamps.get(path, None)
        )
        self.assertIn(path, m.policy_files)
        self.assertEqual(path, m.policy_map.get("policy_A", None))

        path = os.path.join(self.tmp_dir, "policy_2.json")
        self.assertEqual(
            os.path.getmtime(path),
            m.file_timestamps.get(path, None)
        )
        self.assertIn(path, m.policy_files)
        self.assertEqual(path, m.policy_map.get("policy_B", None))
        self.assertEqual(path, m.policy_map.get("policy_C", None))

        self.assertEqual(
            {
                "policy_A": [],
                "policy_B": [],
                "policy_C": []
            },
            m.policy_cache
        )

        self.assertEqual(3, len(m.policy_store.keys()))
        self.assertEqual(
            {
                "groups": {
                    "group_A": {
                        enums.ObjectType.SYMMETRIC_KEY: {
                            enums.Operation.GET: enums.Policy.ALLOW_ALL,
                            enums.Operation.DESTROY: enums.Policy.ALLOW_ALL
                        }
                    }
                }
            },
            m.policy_store.get("policy_A", None)
        )
        self.assertEqual(
            {
                "groups": {
                    "group_B": {
                        enums.ObjectType.SYMMETRIC_KEY: {
                            enums.Operation.GET: enums.Policy.ALLOW_ALL,
                            enums.Operation.LOCATE: enums.Policy.ALLOW_ALL,
                            enums.Operation.DESTROY: enums.Policy.ALLOW_ALL
                        }
                    }
                }
            },
            m.policy_store.get("policy_B", None)
        )
        self.assertEqual(
            {
                "groups": {
                    "group_C": {
                        enums.ObjectType.SYMMETRIC_KEY: {
                            enums.Operation.GET: enums.Policy.ALLOW_ALL,
                            enums.Operation.DESTROY: enums.Policy.DISALLOW_ALL
                        }
                    }
                }
            },
            m.policy_store.get("policy_C", None)
        )

    def test_run_with_adding_new_file_overloading(self):
        """
        Test that the PolicyDirectoryMonitor can load policy files and track
        them properly, even when new policy files are added overwritting
        existing policies while tracking is active.
        """
        m = monitor.PolicyDirectoryMonitor(
            self.tmp_dir,
            multiprocessing.Manager().dict()
        )
        m.logger = mock.MagicMock(logging.Logger)
        m.halt_trigger = mock.MagicMock(multiprocessing.synchronize.Event)
        m.halt_trigger.is_set.side_effect = side_effects(
            [
                False,
                build_write_effect(self.tmp_dir, "policy_3.json", POLICY_2),
                build_write_effect(self.tmp_dir, "policy_4.json", POLICY_3),
                True
            ]
        )

        write_file(self.tmp_dir, "policy_1.json", POLICY_1)
        write_file(self.tmp_dir, "policy_2.json", POLICY_2)

        self.assertEqual({}, m.file_timestamps)
        self.assertEqual({}, m.policy_cache)
        self.assertEqual([], m.policy_files)
        self.assertEqual({}, m.policy_map)
        self.assertEqual([], m.policy_store.keys())

        m.run()

        m.logger.info.assert_any_call(
            "Starting up the operation policy file monitor."
        )
        m.logger.info.assert_any_call(
            "Loading policies for file: {}".format(
                os.path.join(self.tmp_dir, "policy_1.json")
            )
        )
        m.logger.info.assert_any_call("Loading policy: policy_A")
        m.logger.info.assert_any_call(
            "Loading policies for file: {}".format(
                os.path.join(self.tmp_dir, "policy_2.json")
            )
        )
        m.logger.info.assert_any_call("Loading policy: policy_B")
        m.logger.info.assert_any_call("Loading policy: policy_C")
        m.logger.info.assert_any_call(
            "Loading policies for file: {}".format(
                os.path.join(self.tmp_dir, "policy_3.json")
            )
        )
        m.logger.info.assert_any_call("Loading policy: policy_B")
        m.logger.debug.assert_any_call(
            "Policy 'policy_B' overwrites an existing policy."
        )
        m.logger.info.assert_any_call("Loading policy: policy_C")
        m.logger.debug.assert_any_call(
            "Policy 'policy_C' overwrites an existing policy."
        )

        m.logger.info.assert_any_call(
            "Loading policies for file: {}".format(
                os.path.join(self.tmp_dir, "policy_4.json")
            )
        )
        m.logger.info.assert_any_call("Loading policy: policy_B")
        m.logger.debug.assert_any_call(
            "Policy 'policy_B' overwrites an existing policy."
        )

        m.logger.info.assert_any_call(
            "Stopping the operation policy file monitor."
        )

        self.assertEqual(4, len(m.policy_files))
        path = os.path.join(self.tmp_dir, "policy_1.json")
        self.assertEqual(
            os.path.getmtime(path),
            m.file_timestamps.get(path, None)
        )
        self.assertIn(path, m.policy_files)
        self.assertEqual(path, m.policy_map.get("policy_A", None))

        path = os.path.join(self.tmp_dir, "policy_2.json")
        self.assertEqual(
            os.path.getmtime(path),
            m.file_timestamps.get(path, None)
        )
        self.assertIn(path, m.policy_files)

        path = os.path.join(self.tmp_dir, "policy_3.json")
        self.assertEqual(
            os.path.getmtime(path),
            m.file_timestamps.get(path, None)
        )
        self.assertIn(path, m.policy_files)
        self.assertEqual(path, m.policy_map.get("policy_C", None))

        path = os.path.join(self.tmp_dir, "policy_4.json")
        self.assertEqual(
            os.path.getmtime(path),
            m.file_timestamps.get(path, None)
        )
        self.assertIn(path, m.policy_files)
        self.assertEqual(path, m.policy_map.get("policy_B", None))

        self.assertEqual([], m.policy_cache.get("policy_A"))
        cache = m.policy_cache.get("policy_B")
        self.assertEqual(2, len(cache))
        self.assertEqual(
            os.path.join(self.tmp_dir, "policy_2.json"),
            cache[0][1]
        )
        self.assertEqual(
            {
                'groups': {
                    'group_B': {
                        enums.ObjectType.SYMMETRIC_KEY: {
                            enums.Operation.GET:
                                enums.Policy.ALLOW_ALL,
                            enums.Operation.LOCATE:
                                enums.Policy.ALLOW_ALL,
                            enums.Operation.DESTROY:
                                enums.Policy.ALLOW_ALL
                        }
                    }
                }
            },
            cache[0][2]
        )
        self.assertEqual(
            os.path.join(self.tmp_dir, "policy_3.json"),
            cache[1][1]
        )
        self.assertEqual(
            {
                'groups': {
                    'group_B': {
                        enums.ObjectType.SYMMETRIC_KEY: {
                            enums.Operation.GET:
                                enums.Policy.ALLOW_ALL,
                            enums.Operation.LOCATE:
                                enums.Policy.ALLOW_ALL,
                            enums.Operation.DESTROY:
                                enums.Policy.ALLOW_ALL
                        }
                    }
                }
            },
            cache[1][2]
        )
        cache = m.policy_cache.get("policy_C")
        self.assertEqual(1, len(cache))
        self.assertEqual(
            os.path.join(self.tmp_dir, "policy_2.json"),
            cache[0][1]
        )
        self.assertEqual(
            {
                'groups': {
                    'group_C': {
                        enums.ObjectType.SYMMETRIC_KEY: {
                            enums.Operation.GET:
                                enums.Policy.ALLOW_ALL,
                            enums.Operation.DESTROY:
                                enums.Policy.DISALLOW_ALL
                        }
                    }
                }
            },
            cache[0][2]
        )

        self.assertEqual(3, len(m.policy_store.keys()))
        self.assertEqual(
            {
                "groups": {
                    "group_A": {
                        enums.ObjectType.SYMMETRIC_KEY: {
                            enums.Operation.GET: enums.Policy.ALLOW_ALL,
                            enums.Operation.DESTROY: enums.Policy.ALLOW_ALL
                        }
                    }
                }
            },
            m.policy_store.get("policy_A", None)
        )
        self.assertEqual(
            {
                "groups": {
                    "group_B": {
                        enums.ObjectType.SYMMETRIC_KEY: {
                            enums.Operation.GET: enums.Policy.DISALLOW_ALL,
                            enums.Operation.LOCATE: enums.Policy.DISALLOW_ALL,
                            enums.Operation.DESTROY: enums.Policy.DISALLOW_ALL
                        }
                    }
                }
            },
            m.policy_store.get("policy_B", None)
        )
        self.assertEqual(
            {
                "groups": {
                    "group_C": {
                        enums.ObjectType.SYMMETRIC_KEY: {
                            enums.Operation.GET: enums.Policy.ALLOW_ALL,
                            enums.Operation.DESTROY: enums.Policy.DISALLOW_ALL
                        }
                    }
                }
            },
            m.policy_store.get("policy_C", None)
        )

    def test_run_with_adding_new_file_editing_overloading(self):
        """
        Test that the PolicyDirectoryMonitor can load policy files and track
        them properly, even when new policy files are added overwritting
        existing policies while tracking is active.
        """
        m = monitor.PolicyDirectoryMonitor(
            self.tmp_dir,
            multiprocessing.Manager().dict()
        )
        m.logger = mock.MagicMock(logging.Logger)
        m.halt_trigger = mock.MagicMock(multiprocessing.synchronize.Event)
        m.halt_trigger.is_set.side_effect = side_effects(
            [
                False,
                build_write_effect(self.tmp_dir, "policy_3.json", POLICY_2),
                build_write_effect(self.tmp_dir, "policy_4.json", POLICY_3),
                build_delete_effect(self.tmp_dir, "policy_2.json"),
                build_write_effect(self.tmp_dir, "policy_4.json", POLICY_7),
                True
            ]
        )

        write_file(self.tmp_dir, "policy_1.json", POLICY_1)
        write_file(self.tmp_dir, "policy_2.json", POLICY_2)

        self.assertEqual({}, m.file_timestamps)
        self.assertEqual({}, m.policy_cache)
        self.assertEqual([], m.policy_files)
        self.assertEqual({}, m.policy_map)
        self.assertEqual([], m.policy_store.keys())

        m.run()

        m.logger.info.assert_any_call(
            "Starting up the operation policy file monitor."
        )
        m.logger.info.assert_any_call(
            "Loading policies for file: {}".format(
                os.path.join(self.tmp_dir, "policy_1.json")
            )
        )
        m.logger.info.assert_any_call("Loading policy: policy_A")
        m.logger.info.assert_any_call(
            "Loading policies for file: {}".format(
                os.path.join(self.tmp_dir, "policy_2.json")
            )
        )
        m.logger.info.assert_any_call("Loading policy: policy_B")
        m.logger.info.assert_any_call("Loading policy: policy_C")
        m.logger.info.assert_any_call(
            "Loading policies for file: {}".format(
                os.path.join(self.tmp_dir, "policy_3.json")
            )
        )
        m.logger.info.assert_any_call("Loading policy: policy_B")
        m.logger.debug.assert_any_call(
            "Policy 'policy_B' overwrites an existing policy."
        )
        m.logger.info.assert_any_call("Loading policy: policy_C")
        m.logger.debug.assert_any_call(
            "Policy 'policy_C' overwrites an existing policy."
        )
        m.logger.info.assert_any_call(
            "Loading policies for file: {}".format(
                os.path.join(self.tmp_dir, "policy_4.json")
            )
        )
        m.logger.info.assert_any_call("Loading policy: policy_B")
        m.logger.debug.assert_any_call(
            "Policy 'policy_B' overwrites an existing policy."
        )
        m.logger.info.assert_any_call(
            "Removing policies for file: {}".format(
                os.path.join(self.tmp_dir, "policy_2.json")
            )
        )
        m.logger.info.assert_any_call(
            "Loading policies for file: {}".format(
                os.path.join(self.tmp_dir, "policy_4.json")
            )
        )
        m.logger.info.assert_any_call("Loading policy: policy_D")
        m.logger.info.assert_any_call(
            "Stopping the operation policy file monitor."
        )

        self.assertEqual(3, len(m.policy_files))
        path = os.path.join(self.tmp_dir, "policy_1.json")
        self.assertEqual(
            os.path.getmtime(path),
            m.file_timestamps.get(path, None)
        )
        self.assertIn(path, m.policy_files)
        self.assertEqual(path, m.policy_map.get("policy_A", None))

        path = os.path.join(self.tmp_dir, "policy_3.json")
        self.assertEqual(
            os.path.getmtime(path),
            m.file_timestamps.get(path, None)
        )
        self.assertIn(path, m.policy_files)
        self.assertEqual(path, m.policy_map.get("policy_B", None))
        self.assertEqual(path, m.policy_map.get("policy_C", None))

        path = os.path.join(self.tmp_dir, "policy_4.json")
        self.assertEqual(
            os.path.getmtime(path),
            m.file_timestamps.get(path, None)
        )
        self.assertIn(path, m.policy_files)
        self.assertEqual(path, m.policy_map.get("policy_D", None))

        self.assertEqual([], m.policy_cache.get("policy_A"))
        self.assertEqual([], m.policy_cache.get("policy_B"))
        self.assertEqual([], m.policy_cache.get("policy_C"))

        self.assertEqual(4, len(m.policy_store.keys()))
        self.assertEqual(
            {
                "groups": {
                    "group_A": {
                        enums.ObjectType.SYMMETRIC_KEY: {
                            enums.Operation.GET: enums.Policy.ALLOW_ALL,
                            enums.Operation.DESTROY: enums.Policy.ALLOW_ALL
                        }
                    }
                }
            },
            m.policy_store.get("policy_A", None)
        )
        self.assertEqual(
            {
                "groups": {
                    "group_B": {
                        enums.ObjectType.SYMMETRIC_KEY: {
                            enums.Operation.GET: enums.Policy.ALLOW_ALL,
                            enums.Operation.LOCATE: enums.Policy.ALLOW_ALL,
                            enums.Operation.DESTROY: enums.Policy.ALLOW_ALL
                        }
                    }
                }
            },
            m.policy_store.get("policy_B", None)
        )
        self.assertEqual(
            {
                "groups": {
                    "group_C": {
                        enums.ObjectType.SYMMETRIC_KEY: {
                            enums.Operation.GET: enums.Policy.ALLOW_ALL,
                            enums.Operation.DESTROY: enums.Policy.DISALLOW_ALL
                        }
                    }
                }
            },
            m.policy_store.get("policy_C", None)
        )
        self.assertEqual(
            {
                "groups": {
                    "group_D": {
                        enums.ObjectType.SYMMETRIC_KEY: {
                            enums.Operation.GET: enums.Policy.DISALLOW_ALL,
                            enums.Operation.LOCATE: enums.Policy.DISALLOW_ALL,
                            enums.Operation.DESTROY: enums.Policy.DISALLOW_ALL
                        }
                    }
                }
            },
            m.policy_store.get("policy_D", None)
        )

    def test_run_without_live_monitoring(self):
        """
        Test that the PolicyDirectoryMonitor can load policy files and track
        them properly even when operating in a one-shot scanning mode.
        """
        m = monitor.PolicyDirectoryMonitor(
            self.tmp_dir,
            multiprocessing.Manager().dict(),
            live_monitoring=False
        )
        m.logger = mock.MagicMock(logging.Logger)
        m.halt_trigger = mock.MagicMock(multiprocessing.synchronize.Event)
        m.halt_trigger.is_set.side_effect = [False, True]

        write_file(self.tmp_dir, "policy_1.json", POLICY_1)
        write_file(self.tmp_dir, "policy_2.json", POLICY_2)

        self.assertEqual({}, m.file_timestamps)
        self.assertEqual({}, m.policy_cache)
        self.assertEqual([], m.policy_files)
        self.assertEqual({}, m.policy_map)
        self.assertEqual([], m.policy_store.keys())

        m.run()

        m.logger.info.assert_any_call(
            "Loading policies for file: {}".format(
                os.path.join(self.tmp_dir, "policy_1.json")
            )
        )
        m.logger.info.assert_any_call("Loading policy: policy_A")
        m.logger.info.assert_any_call(
            "Loading policies for file: {}".format(
                os.path.join(self.tmp_dir, "policy_2.json")
            )
        )
        m.logger.info.assert_any_call("Loading policy: policy_B")
        m.logger.info.assert_any_call("Loading policy: policy_C")

        self.assertEqual(2, len(m.policy_files))
        path = os.path.join(self.tmp_dir, "policy_1.json")
        self.assertEqual(
            os.path.getmtime(path),
            m.file_timestamps.get(path, None)
        )
        self.assertIn(path, m.policy_files)
        self.assertEqual(path, m.policy_map.get("policy_A", None))

        path = os.path.join(self.tmp_dir, "policy_2.json")
        self.assertEqual(
            os.path.getmtime(path),
            m.file_timestamps.get(path, None)
        )
        self.assertIn(path, m.policy_files)
        self.assertEqual(path, m.policy_map.get("policy_B", None))
        self.assertEqual(path, m.policy_map.get("policy_C", None))

        self.assertEqual(
            {
                "policy_A": [],
                "policy_B": [],
                "policy_C": []
            },
            m.policy_cache
        )

        self.assertEqual(3, len(m.policy_store.keys()))
        self.assertEqual(
            {
                "groups": {
                    "group_A": {
                        enums.ObjectType.SYMMETRIC_KEY: {
                            enums.Operation.GET: enums.Policy.ALLOW_ALL,
                            enums.Operation.DESTROY: enums.Policy.ALLOW_ALL
                        }
                    }
                }
            },
            m.policy_store.get("policy_A", None)
        )
        self.assertEqual(
            {
                "groups": {
                    "group_B": {
                        enums.ObjectType.SYMMETRIC_KEY: {
                            enums.Operation.GET: enums.Policy.ALLOW_ALL,
                            enums.Operation.LOCATE: enums.Policy.ALLOW_ALL,
                            enums.Operation.DESTROY: enums.Policy.ALLOW_ALL
                        }
                    }
                }
            },
            m.policy_store.get("policy_B", None)
        )
        self.assertEqual(
            {
                "groups": {
                    "group_C": {
                        enums.ObjectType.SYMMETRIC_KEY: {
                            enums.Operation.GET: enums.Policy.ALLOW_ALL,
                            enums.Operation.DESTROY: enums.Policy.DISALLOW_ALL
                        }
                    }
                }
            },
            m.policy_store.get("policy_C", None)
        )

    def test_initialize_tracking_structures(self):
        """
        Test that the PolicyDirectoryMonitor can correctly initialize/reset the
        various tracking structures used for file monitoring.
        """
        m = monitor.PolicyDirectoryMonitor(
            self.tmp_dir,
            multiprocessing.Manager().dict()
        )

        m.file_timestamps["a"] = 1234
        m.policy_cache["a"] = (123.12, "b", {"c": 2})
        m.policy_files = ["a", "b"]
        m.policy_map["a"] = "b"
        m.policy_store["a"] = {"c": 2}
        m.policy_store["default"] = {"c": 3}

        m.initialize_tracking_structures()

        self.assertEqual({}, m.file_timestamps)
        self.assertEqual({}, m.policy_cache)
        self.assertEqual([], m.policy_files)
        self.assertEqual({}, m.policy_map)
        self.assertEqual(["default"], m.policy_store.keys())
        self.assertEqual({"c": 3}, m.policy_store.get("default"))

    def test_disassociate_policy_and_file(self):
        """
        Test that the PolicyDirectoryMonitor can correctly unlink a policy and
        a policy file in its tracking structures.
        """
        m = monitor.PolicyDirectoryMonitor(
            self.tmp_dir,
            multiprocessing.Manager().dict()
        )

        m.policy_cache = {
            "policy_A": [
                (
                    1480043060.870089,
                    os.path.join(self.tmp_dir, "policy_1.json"),
                    {}
                ),
                (
                    1480043062.02171,
                    os.path.join(self.tmp_dir, "policy_2.json"),
                    {}
                ),
                (
                    1480043062.645776,
                    os.path.join(self.tmp_dir, "policy_1.json"),
                    {}
                ),
                (
                    1480043063.453713,
                    os.path.join(self.tmp_dir, "policy_3.json"),
                    {}
                )
            ],
            "policy_B": [
                (
                    1480043123.65311,
                    os.path.join(self.tmp_dir, "policy_1.json"),
                    {}
                )
            ]
        }

        m.disassociate_policy_and_file(
            "policy_A",
            os.path.join(self.tmp_dir, "policy_1.json")
        )

        self.assertEqual(
            [
                (
                    1480043062.02171,
                    os.path.join(self.tmp_dir, "policy_2.json"),
                    {}
                ),
                (
                    1480043063.453713,
                    os.path.join(self.tmp_dir, "policy_3.json"),
                    {}
                )
            ],
            m.policy_cache.get("policy_A", [])
        )
        self.assertEqual(
            [
                (
                    1480043123.65311,
                    os.path.join(self.tmp_dir, "policy_1.json"),
                    {}
                )
            ],
            m.policy_cache.get("policy_B", [])
        )

    def test_restore_or_delete_policy_restore(self):
        """
        Test that the PolicyDirectoryMonitor can correctly restore policy data
        upon a policy file change.
        """
        m = monitor.PolicyDirectoryMonitor(
            self.tmp_dir,
            multiprocessing.Manager().dict()
        )
        m.logger = mock.MagicMock(logging.Logger)

        m.policy_cache = {
            "policy_A": [
                (
                    1480043060.870089,
                    os.path.join(self.tmp_dir, "policy_1.json"),
                    {'{"policy_1"}'}
                ),
                (
                    1480043062.02171,
                    os.path.join(self.tmp_dir, "policy_2.json"),
                    {'{"policy_2"}'}
                ),
                (
                    1480043063.453713,
                    os.path.join(self.tmp_dir, "policy_3.json"),
                    {'{"policy_3"}'}
                )
            ]
        }
        m.policy_store["policy_A"] = {'{"policy_4"}'}
        m.policy_map["policy_A"] = os.path.join(self.tmp_dir, "policy_4.json")

        m.restore_or_delete_policy("policy_A")

        m.logger.info.assert_not_called()
        self.assertEqual(
            [
                (
                    1480043060.870089,
                    os.path.join(self.tmp_dir, "policy_1.json"),
                    {'{"policy_1"}'}
                ),
                (
                    1480043062.02171,
                    os.path.join(self.tmp_dir, "policy_2.json"),
                    {'{"policy_2"}'}
                )
            ],
            m.policy_cache.get("policy_A", [])
        )
        self.assertEqual(
            {'{"policy_3"}'},
            m.policy_store.get("policy_A", {})
        )
        self.assertEqual(
            os.path.join(self.tmp_dir, "policy_3.json"),
            m.policy_map.get("policy_A", None)
        )

    def test_restore_or_delete_policy_delete(self):
        """
        Test that the PolicyDirectoryMonitor can correctly delete policy data
        upon a policy file change.
        """
        m = monitor.PolicyDirectoryMonitor(
            self.tmp_dir,
            multiprocessing.Manager().dict()
        )
        m.logger = mock.MagicMock(logging.Logger)

        m.policy_cache = {
            "policy_A": []
        }
        m.policy_store["policy_A"] = {'{"policy_4"}'}
        m.policy_map["policy_A"] = os.path.join(self.tmp_dir, "policy_4.json")

        m.restore_or_delete_policy("policy_A")

        m.logger.info.assert_called_once_with("Removing policy: policy_A")
        self.assertNotIn("policy_A", m.policy_cache.keys())
        self.assertNotIn("policy_A", m.policy_store.keys())
        self.assertNotIn("policy_A", m.policy_map.keys())



# Copyright (c) 2015 The Johns Hopkins University/Applied Physics Laboratory
# All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import logging
import sys

from kmip.core import enums
from kmip.demos import utils

from kmip.pie import client
from kmip.pie import objects


if __name__ == '__main__':
    logger = utils.build_console_logger(logging.INFO)

    parser = utils.build_cli_parser()
    opts, args = parser.parse_args(sys.argv[1:])

    config = opts.config

    algorithm = enums.CryptographicAlgorithm.RSA
    length = 2048
    value = (
        b'\x30\x82\x01\x0A\x02\x82\x01\x01\x00\xAB\x7F\x16\x1C\x00\x42\x49'
        b'\x6C\xCD\x6C\x6D\x4D\xAD\xB9\x19\x97\x34\x35\x35\x77\x76\x00\x3A'
        b'\xCF\x54\xB7\xAF\x1E\x44\x0A\xFB\x80\xB6\x4A\x87\x55\xF8\x00\x2C'
        b'\xFE\xBA\x6B\x18\x45\x40\xA2\xD6\x60\x86\xD7\x46\x48\x34\x6D\x75'
        b'\xB8\xD7\x18\x12\xB2\x05\x38\x7C\x0F\x65\x83\xBC\x4D\x7D\xC7\xEC'
        b'\x11\x4F\x3B\x17\x6B\x79\x57\xC4\x22\xE7\xD0\x3F\xC6\x26\x7F\xA2'
        b'\xA6\xF8\x9B\x9B\xEE\x9E\x60\xA1\xD7\xC2\xD8\x33\xE5\xA5\xF4\xBB'
        b'\x0B\x14\x34\xF4\xE7\x95\xA4\x11\x00\xF8\xAA\x21\x49\x00\xDF\x8B'
        b'\x65\x08\x9F\x98\x13\x5B\x1C\x67\xB7\x01\x67\x5A\xBD\xBC\x7D\x57'
        b'\x21\xAA\xC9\xD1\x4A\x7F\x08\x1F\xCE\xC8\x0B\x64\xE8\xA0\xEC\xC8'
        b'\x29\x53\x53\xC7\x95\x32\x8A\xBF\x70\xE1\xB4\x2E\x7B\xB8\xB7\xF4'
        b'\xE8\xAC\x8C\x81\x0C\xDB\x66\xE3\xD2\x11\x26\xEB\xA8\xDA\x7D\x0C'
        b'\xA3\x41\x42\xCB\x76\xF9\x1F\x01\x3D\xA8\x09\xE9\xC1\xB7\xAE\x64'
        b'\xC5\x41\x30\xFB\xC2\x1D\x80\xE9\xC2\xCB\x06\xC5\xC8\xD7\xCC\xE8'
        b'\x94\x6A\x9A\xC9\x9B\x1C\x28\x15\xC3\x61\x2A\x29\xA8\x2D\x73\xA1'
        b'\xF9\x93\x74\xFE\x30\xE5\x49\x51\x66\x2A\x6E\xDA\x29\xC6\xFC\x41'
        b'\x13\x35\xD5\xDC\x74\x26\xB0\xF6\x05\x02\x03\x01\x00\x01')
    format_type = enums.KeyFormatType.X_509
    usage_mask = [enums.CryptographicUsageMask.VERIFY]
    name = 'Demo Public Key'

    key = objects.PublicKey(
        algorithm, length, value, format_type, usage_mask, name)
    key.operation_policy_name = opts.operation_policy_name

    # Build the client and connect to the server
    with client.ProxyKmipClient(
            config=config,
            config_file=opts.config_file
    ) as client:
        try:
            uid = client.register(key)
            logger.info("Successfully registered public key with ID: "
                        "{0}".format(uid))
        except Exception as e:
            logger.error(e)



# Copyright (c) 2017 The Johns Hopkins University/Applied Physics Laboratory
# All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import logging
import sys

from kmip.core import enums
from kmip.demos import utils
from kmip.pie import client


if __name__ == '__main__':
    logger = utils.build_console_logger(logging.INFO)

    # Build and parse arguments
    parser = utils.build_cli_parser(enums.Operation.DERIVE_KEY)
    opts, args = parser.parse_args(sys.argv[1:])
    config = opts.config

    # Build the client and connect to the server
    with client.ProxyKmipClient(
            config=config,
            config_file=opts.config_file
    ) as client:
        # Create keys to use for derivation
        try:
            key_id = client.create(
                enums.CryptographicAlgorithm.AES,
                128,
                cryptographic_usage_mask=[
                    enums.CryptographicUsageMask.DERIVE_KEY
                ]
            )
            logger.info("Successfully created a new derivation key.")
            logger.info("Secret ID: {0}".format(key_id))
        except Exception as e:
            logger.error(e)
            sys.exit(-1)

        # Derive a new secret via PBKDF2.
        try:
            secret_id = client.derive_key(
                enums.ObjectType.SYMMETRIC_KEY,
                [key_id],
                enums.DerivationMethod.PBKDF2,
                {
                    'cryptographic_parameters': {
                        'hashing_algorithm': enums.HashingAlgorithm.SHA_1
                    },
                    'salt': b'salt',
                    'iteration_count': 4096
                },
                cryptographic_length=160,
                cryptographic_algorithm=enums.CryptographicAlgorithm.AES
            )
            logger.info("Successfully derived a new secret via PBKDF2.")
            logger.info("Secret ID: {0}".format(secret_id))
        except Exception as e:
            logger.error(e)

        # Derive a new secret via encryption.
        try:
            secret_id = client.derive_key(
                enums.ObjectType.SECRET_DATA,
                [key_id],
                enums.DerivationMethod.ENCRYPT,
                {
                    'cryptographic_parameters': {
                        'block_cipher_mode': enums.BlockCipherMode.CBC,
                        'padding_method': enums.PaddingMethod.PKCS5,
                        'cryptographic_algorithm':
                            enums.CryptographicAlgorithm.BLOWFISH
                    },
                    'initialization_vector': (
                        b'\xFE\xDC\xBA\x98\x76\x54\x32\x10'
                    ),
                    'derivation_data': (
                        b'\x37\x36\x35\x34\x33\x32\x31\x30'
                        b'\x4E\x6F\x77\x20\x69\x73\x20\x74'
                        b'\x68\x65\x20\x74\x69\x6D\x65\x20'
                        b'\x66\x6F\x72\x20\x00'
                    )
                },
                cryptographic_length=256
            )
            logger.info("Successfully derived a new secret via encryption.")
            logger.info("Secret ID: {0}".format(secret_id))
        except Exception as e:
            logger.error(e)

        # Derive a new secret via HMAC.
        try:
            secret_id = client.derive_key(
                enums.ObjectType.SYMMETRIC_KEY,
                [key_id],
                enums.DerivationMethod.HMAC,
                {
                    'cryptographic_parameters': {
                        'hashing_algorithm': enums.HashingAlgorithm.SHA_256
                    },
                    'derivation_data': (
                        b'\xF0\xF1\xF2\xF3\xF4\xF5\xF6\xF7'
                        b'\xF8\xF9'
                    ),
                    'salt': (
                        b'\x00\x01\x02\x03\x04\x05\x06\x07'
                        b'\x08\x09\x0A\x0B\x0C'
                    )
                },
                cryptographic_length=64,
                cryptographic_algorithm=enums.CryptographicAlgorithm.RC4
            )
            logger.info("Successfully derived a new secret via HMAC.")
            logger.info("Secret ID: {0}".format(secret_id))
        except Exception as e:
            logger.error(e)

        # Derive a new secret via hashing.
        try:
            secret_id = client.derive_key(
                enums.ObjectType.SECRET_DATA,
                [key_id],
                enums.DerivationMethod.HASH,
                {
                    'cryptographic_parameters': {
                        'hashing_algorithm': enums.HashingAlgorithm.MD5
                    }
                },
                cryptographic_length=128
            )
            logger.info("Successfully derived a new secret via hashing.")
            logger.info("Secret ID: {0}".format(secret_id))
        except Exception as e:
            logger.error(e)

        # Derive a new secret via NIST 800 108-C.
        try:
            secret_id = client.derive_key(
                enums.ObjectType.SYMMETRIC_KEY,
                [key_id],
                enums.DerivationMethod.NIST800_108_C,
                {
                    'cryptographic_parameters': {
                        'hashing_algorithm': enums.HashingAlgorithm.SHA_1
                    },
                    'derivation_data': (
                        b'\x8e\x34\x7e\xf5\x5d\x5f\x5e\x99'
                        b'\xea\xb6\xde\x70\x6b\x51\xde\x7c'
                        b'\xe0\x04\xf3\x88\x28\x89\xe2\x59'
                        b'\xff\x4e\x5c\xff\x10\x21\x67\xa5'
                        b'\xa4\xbd\x71\x15\x78\xd4\xce\x17'
                        b'\xdd\x9a\xbe\x56\xe5\x1c\x1f\x2d'
                        b'\xf9\x50\xe2\xfc\x81\x2e\xc1\xb2'
                        b'\x17\xca\x08\xd6'
                    )
                },
                cryptographic_length=128,
                cryptographic_algorithm=enums.CryptographicAlgorithm.AES
            )
            logger.info(
                "Successfully derived a new secret via NIST 800 108-C."
            )
            logger.info("Secret ID: {0}".format(secret_id))
        except Exception as e:
            logger.error(e)



# Copyright (c) 2014 The Johns Hopkins University/Applied Physics Laboratory
# All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from kmip.core import attributes
from kmip.core import enums
from kmip.core import primitives
from kmip.core import utils


class AttributeValueFactory(object):

    def create_attribute_value(self, name, value):
        # Switch on the name of the attribute
        if name is enums.AttributeType.UNIQUE_IDENTIFIER:
            return attributes.UniqueIdentifier(value)
        elif name is enums.AttributeType.NAME:
            return self._create_name(value)
        elif name is enums.AttributeType.OBJECT_TYPE:
            return attributes.ObjectType(value)
        elif name is enums.AttributeType.CRYPTOGRAPHIC_ALGORITHM:
            return attributes.CryptographicAlgorithm(value)
        elif name is enums.AttributeType.CRYPTOGRAPHIC_LENGTH:
            return self._create_cryptographic_length(value)
        elif name is enums.AttributeType.CRYPTOGRAPHIC_PARAMETERS:
            return self._create_cryptographic_parameters(value)
        elif name is enums.AttributeType.CRYPTOGRAPHIC_DOMAIN_PARAMETERS:
            raise NotImplementedError()
        elif name is enums.AttributeType.CERTIFICATE_TYPE:
            return primitives.Enumeration(
                enums.CertificateType,
                value=value,
                tag=enums.Tags.CERTIFICATE_TYPE
            )
        elif name is enums.AttributeType.CERTIFICATE_LENGTH:
            return primitives.Integer(value, enums.Tags.CERTIFICATE_LENGTH)
        elif name is enums.AttributeType.X_509_CERTIFICATE_IDENTIFIER:
            raise NotImplementedError()
        elif name is enums.AttributeType.X_509_CERTIFICATE_SUBJECT:
            raise NotImplementedError()
        elif name is enums.AttributeType.X_509_CERTIFICATE_ISSUER:
            raise NotImplementedError()
        elif name is enums.AttributeType.CERTIFICATE_IDENTIFIER:
            raise NotImplementedError()
        elif name is enums.AttributeType.CERTIFICATE_SUBJECT:
            raise NotImplementedError()
        elif name is enums.AttributeType.CERTIFICATE_ISSUER:
            raise NotImplementedError()
        elif name is enums.AttributeType.DIGITAL_SIGNATURE_ALGORITHM:
            raise NotImplementedError()
        elif name is enums.AttributeType.DIGEST:
            return attributes.Digest()
        elif name is enums.AttributeType.OPERATION_POLICY_NAME:
            return attributes.OperationPolicyName(value)
        elif name is enums.AttributeType.CRYPTOGRAPHIC_USAGE_MASK:
            return self._create_cryptographic_usage_mask(value)
        elif name is enums.AttributeType.LEASE_TIME:
            return primitives.Interval(value, enums.Tags.LEASE_TIME)
        elif name is enums.AttributeType.USAGE_LIMITS:
            raise NotImplementedError()
        elif name is enums.AttributeType.STATE:
            return attributes.State(value)
        elif name is enums.AttributeType.INITIAL_DATE:
            return primitives.DateTime(value, enums.Tags.INITIAL_DATE)
        elif name is enums.AttributeType.ACTIVATION_DATE:
            return primitives.DateTime(value, enums.Tags.ACTIVATION_DATE)
        elif name is enums.AttributeType.PROCESS_START_DATE:
            return primitives.DateTime(value, enums.Tags.PROCESS_START_DATE)
        elif name is enums.AttributeType.PROTECT_STOP_DATE:
            return primitives.DateTime(value, enums.Tags.PROTECT_STOP_DATE)
        elif name is enums.AttributeType.DEACTIVATION_DATE:
            return primitives.DateTime(value, enums.Tags.DEACTIVATION_DATE)
        elif name is enums.AttributeType.DESTROY_DATE:
            return primitives.DateTime(value, enums.Tags.DESTROY_DATE)
        elif name is enums.AttributeType.COMPROMISE_OCCURRENCE_DATE:
            return primitives.DateTime(
                value, enums.Tags.COMPROMISE_OCCURRENCE_DATE)
        elif name is enums.AttributeType.COMPROMISE_DATE:
            return primitives.DateTime(value, enums.Tags.COMPROMISE_DATE)
        elif name is enums.AttributeType.REVOCATION_REASON:
            raise NotImplementedError()
        elif name is enums.AttributeType.ARCHIVE_DATE:
            return primitives.DateTime(value, enums.Tags.ARCHIVE_DATE)
        elif name is enums.AttributeType.OBJECT_GROUP:
            return primitives.TextString(value, enums.Tags.OBJECT_GROUP)
        elif name is enums.AttributeType.FRESH:
            return primitives.Boolean(value, enums.Tags.FRESH)
        elif name is enums.AttributeType.LINK:
            raise NotImplementedError()
        elif name is enums.AttributeType.APPLICATION_SPECIFIC_INFORMATION:
            return self._create_application_specific_information(value)
        elif name is enums.AttributeType.CONTACT_INFORMATION:
            return self._create_contact_information(value)
        elif name is enums.AttributeType.LAST_CHANGE_DATE:
            return primitives.DateTime(value, enums.Tags.LAST_CHANGE_DATE)
        elif name is enums.AttributeType.SENSITIVE:
            return primitives.Boolean(value, enums.Tags.SENSITIVE)
        elif name is enums.AttributeType.CUSTOM_ATTRIBUTE:
            return attributes.CustomAttribute(value)
        else:
            if not isinstance(name, str):
                raise ValueError('Unrecognized attribute type: '
                                 '{0}'.format(name))
            elif name.startswith('x-'):
                # Custom attribute indicated
                return attributes.CustomAttribute(value)

    def create_attribute_value_by_enum(self, enum, value):
        # Switch on the name of the attribute
        if enum is enums.Tags.UNIQUE_IDENTIFIER:
            return attributes.UniqueIdentifier(value)
        elif enum is enums.Tags.NAME:
            return self._create_name(value)
        elif enum is enums.Tags.OBJECT_TYPE:
            return attributes.ObjectType(value)
        elif enum is enums.Tags.CRYPTOGRAPHIC_ALGORITHM:
            return attributes.CryptographicAlgorithm(value)
        elif enum is enums.Tags.CRYPTOGRAPHIC_LENGTH:
            return self._create_cryptographic_length(value)
        elif enum is enums.Tags.CRYPTOGRAPHIC_PARAMETERS:
            return self._create_cryptographic_parameters(value)
        elif enum is enums.Tags.CRYPTOGRAPHIC_DOMAIN_PARAMETERS:
            raise NotImplementedError()
        elif enum is enums.Tags.CERTIFICATE_TYPE:
            raise NotImplementedError()
        elif enum is enums.Tags.CERTIFICATE_LENGTH:
            return primitives.Integer(value, enums.Tags.CERTIFICATE_LENGTH)
        elif enum is enums.Tags.X_509_CERTIFICATE_IDENTIFIER:
            raise NotImplementedError()
        elif enum is enums.Tags.X_509_CERTIFICATE_SUBJECT:
            raise NotImplementedError()
        elif enum is enums.Tags.X_509_CERTIFICATE_ISSUER:
            raise NotImplementedError()
        elif enum is enums.Tags.CERTIFICATE_IDENTIFIER:
            raise NotImplementedError()
        elif enum is enums.Tags.CERTIFICATE_SUBJECT:
            raise NotImplementedError()
        elif enum is enums.Tags.CERTIFICATE_ISSUER:
            raise NotImplementedError()
        elif enum is enums.Tags.DIGITAL_SIGNATURE_ALGORITHM:
            raise NotImplementedError()
        elif enum is enums.Tags.DIGEST:
            return attributes.Digest()
        elif enum is enums.Tags.OPERATION_POLICY_NAME:
            return attributes.OperationPolicyName(value)
        elif enum is enums.Tags.CRYPTOGRAPHIC_USAGE_MASK:
            return self._create_cryptographic_usage_mask(value)
        elif enum is enums.Tags.LEASE_TIME:
            return primitives.Interval(value, enums.Tags.LEASE_TIME)
        elif enum is enums.Tags.USAGE_LIMITS:
            raise NotImplementedError()
        elif enum is enums.Tags.STATE:
            return attributes.State(value)
        elif enum is enums.Tags.INITIAL_DATE:
            return primitives.DateTime(value, enums.Tags.INITIAL_DATE)
        elif enum is enums.Tags.ACTIVATION_DATE:
            return primitives.DateTime(value, enums.Tags.ACTIVATION_DATE)
        elif enum is enums.Tags.PROCESS_START_DATE:
            return primitives.DateTime(value, enums.Tags.PROCESS_START_DATE)
        elif enum is enums.Tags.PROTECT_STOP_DATE:
            return primitives.DateTime(value, enums.Tags.PROTECT_STOP_DATE)
        elif enum is enums.Tags.DEACTIVATION_DATE:
            return primitives.DateTime(value, enums.Tags.DEACTIVATION_DATE)
        elif enum is enums.Tags.DESTROY_DATE:
            return primitives.DateTime(value, enums.Tags.DESTROY_DATE)
        elif enum is enums.Tags.COMPROMISE_OCCURRENCE_DATE:
            return primitives.DateTime(
                value, enums.Tags.COMPROMISE_OCCURRENCE_DATE)
        elif enum is enums.Tags.COMPROMISE_DATE:
            return primitives.DateTime(value, enums.Tags.COMPROMISE_DATE)
        elif enum is enums.Tags.REVOCATION_REASON:
            raise NotImplementedError()
        elif enum is enums.Tags.ARCHIVE_DATE:
            return primitives.DateTime(value, enums.Tags.ARCHIVE_DATE)
        elif enum is enums.Tags.OBJECT_GROUP:
            return primitives.TextString(value, enums.Tags.OBJECT_GROUP)
            return self._create_object_group(value)
        elif enum is enums.Tags.FRESH:
            return primitives.Boolean(value, enums.Tags.FRESH)
        elif enum is enums.Tags.LINK:
            raise NotImplementedError()
        elif enum is enums.Tags.APPLICATION_SPECIFIC_INFORMATION:
            return self._create_application_specific_information(value)
        elif enum is enums.Tags.CONTACT_INFORMATION:
            return self._create_contact_information(value)
        elif enum is enums.Tags.LAST_CHANGE_DATE:
            return primitives.DateTime(value, enums.Tags.LAST_CHANGE_DATE)
        elif enum is enums.Tags.SENSITIVE:
            return primitives.Boolean(value, enums.Tags.SENSITIVE)
        elif enum is enums.Tags.CUSTOM_ATTRIBUTE:
            return attributes.CustomAttribute(value)
        else:
            raise ValueError("Unrecognized attribute type: {}".format(enum))

    def _create_name(self, name):
        if name is not None:
            if isinstance(name, attributes.Name):
                return attributes.Name.create(name.name_value, name.name_type)

            elif isinstance(name, str):
                return attributes.Name.create(
                            name,
                            enums.NameType.UNINTERPRETED_TEXT_STRING
                        )
            else:
                raise ValueError('Unrecognized attribute type: '
                                 '{0}'.format(name))
        else:
            return attributes.Name()

    def _create_cryptographic_length(self, length):
        if length is not None and not isinstance(length, int):
            msg = utils.build_er_error(attributes.CryptographicLength,
                                       'constructor argument type', int,
                                       type(length))
            raise TypeError(msg)

        return attributes.CryptographicLength(length)

    def _create_cryptographic_parameters(self, params):
        if params is None:
            params = {}

        if isinstance(params, dict):
            return attributes.CryptographicParameters(
                block_cipher_mode=params.get('block_cipher_mode', None),
                padding_method=params.get('padding_method', None),
                hashing_algorithm=params.get('hashing_algorithm', None),
                key_role_type=params.get('key_role_type', None),
                digital_signature_algorithm=params.get(
                    'digital_signature_algorithm',
                    None
                ),
                cryptographic_algorithm=params.get(
                    'cryptographic_algorithm',
                    None
                ),
                random_iv=params.get('random_iv', None),
                iv_length=params.get('iv_length', None),
                tag_length=params.get('tag_length', None),
                fixed_field_length=params.get('fixed_field_length', None),
                invocation_field_length=params.get(
                    'invocation_field_length',
                    None
                ),
                counter_length=params.get('counter_length', None),
                initial_counter_value=params.get(
                    'initial_counter_value',
                    None
                )
            )
        else:
            raise TypeError("cryptographic parameters must be a dict")

    def _create_cryptographic_usage_mask(self, flags):
        mask = None
        if flags is not None:
            mask = 0
            for flag in flags:
                mask |= flag.value

        return attributes.CryptographicUsageMask(mask)

    def _create_application_specific_information(self, info):
        if info:
            return attributes.ApplicationSpecificInformation(
                application_namespace=info.get("application_namespace"),
                application_data=info.get("application_data")
            )
        else:
            return attributes.ApplicationSpecificInformation()

    def _create_contact_information(self, info):
        if info is None:
            return attributes.ContactInformation()
        else:
            if not isinstance(info, str):
                msg = utils.build_er_error(attributes.ContactInformation,
                                           'constructor argument type', str,
                                           type(info))
                raise TypeError(msg)

            return attributes.ContactInformation(info)



# Copyright (c) 2014 The Johns Hopkins University/Applied Physics Laboratory
# All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from kmip.core.enums import CredentialType
from kmip.core.enums import Operation
from kmip.core.enums import ResultStatus

from kmip.core.factories.attributes import AttributeFactory
from kmip.core.factories.credentials import CredentialFactory

from kmip.demos import utils

from kmip.services.kmip_client import KMIPProxy

import logging
import sys


if __name__ == '__main__':
    logger = utils.build_console_logger(logging.INFO)

    # Build and parse arguments
    parser = utils.build_cli_parser(Operation.DESTROY)
    opts, args = parser.parse_args(sys.argv[1:])

    username = opts.username
    password = opts.password
    config = opts.config
    uuid = opts.uuid

    # Exit early if the UUID is not specified
    if uuid is None:
        logger.error('No UUID provided, exiting early from demo')
        sys.exit()

    attribute_factory = AttributeFactory()
    credential_factory = CredentialFactory()

    # Build the KMIP server account credentials
    # TODO (peter-hamilton) Move up into KMIPProxy
    if (username is None) and (password is None):
        credential = None
    else:
        credential_type = CredentialType.USERNAME_AND_PASSWORD
        credential_value = {'Username': username,
                            'Password': password}
        credential = credential_factory.create_credential(credential_type,
                                                          credential_value)
    # Build the client and connect to the server
    client = KMIPProxy(config=config, config_file=opts.config_file)
    client.open()

    # Destroy the SYMMETRIC_KEY object
    result = client.destroy(uuid, credential)
    client.close()

    # Display operation results
    logger.info('destroy() result status: {0}'.format(
        result.result_status.value))

    if result.result_status.value == ResultStatus.SUCCESS:
        logger.info('destroyed UUID: {0}'.format(result.uuid.value))
    else:
        logger.info('destroy() result reason: {0}'.format(
            result.result_reason.value))
        logger.info('destroy() result message: {0}'.format(
            result.result_message.value))



#!/usr/bin/env python

from cryptography import x509
from cryptography.hazmat import backends
from cryptography.hazmat.primitives import hashes
from cryptography.hazmat.primitives import serialization
from cryptography.hazmat.primitives.asymmetric import rsa

import datetime


def create_rsa_private_key(key_size=2048, public_exponent=65537):
    private_key = rsa.generate_private_key(
        public_exponent=public_exponent,
        key_size=key_size,
        backend=backends.default_backend()
    )
    return private_key


def create_self_signed_certificate(subject_name, private_key, days_valid=365):
    subject = x509.Name([
        x509.NameAttribute(x509.NameOID.ORGANIZATION_NAME, u"Test, Inc."),
        x509.NameAttribute(x509.NameOID.COMMON_NAME, subject_name)
    ])
    certificate = x509.CertificateBuilder().subject_name(
        subject
    ).issuer_name(
        subject
    ).public_key(
        private_key.public_key()
    ).serial_number(
        x509.random_serial_number()
    ).add_extension(
        x509.BasicConstraints(ca=True, path_length=None), critical=True
    ).not_valid_before(
        datetime.datetime.utcnow()
    ).not_valid_after(
        datetime.datetime.utcnow() + datetime.timedelta(days=days_valid)
    ).sign(private_key, hashes.SHA256(), backends.default_backend())

    return certificate


def create_certificate(subject_name,
                       private_key,
                       signing_certificate,
                       signing_key,
                       days_valid=365,
                       client_auth=False):
    subject = x509.Name([
        x509.NameAttribute(x509.NameOID.ORGANIZATION_NAME, u"Test, Inc."),
        x509.NameAttribute(x509.NameOID.COMMON_NAME, subject_name)
    ])
    builder = x509.CertificateBuilder().subject_name(
        subject
    ).issuer_name(
        signing_certificate.subject
    ).public_key(
        private_key.public_key()
    ).serial_number(
        x509.random_serial_number()
    ).not_valid_before(
        datetime.datetime.utcnow()
    ).not_valid_after(
        datetime.datetime.utcnow() + datetime.timedelta(days=days_valid)
    )

    if client_auth:
        builder = builder.add_extension(
            x509.ExtendedKeyUsage([x509.ExtendedKeyUsageOID.CLIENT_AUTH]),
            critical=True
        )

    certificate = builder.sign(
        signing_key,
        hashes.SHA256(),
        backends.default_backend()
    )
    return certificate


def main():
    root_key = create_rsa_private_key()
    root_certificate = create_self_signed_certificate(
        u"Root CA",
        root_key
    )

    server_key = create_rsa_private_key()
    server_certificate = create_certificate(
        u"Server Certificate",
        server_key,
        root_certificate,
        root_key
    )

    john_doe_client_key = create_rsa_private_key()
    john_doe_client_certificate = create_certificate(
        u"John Doe",
        john_doe_client_key,
        root_certificate,
        root_key,
        client_auth=True
    )
    jane_doe_client_key = create_rsa_private_key()
    jane_doe_client_certificate = create_certificate(
        u"Jane Doe",
        jane_doe_client_key,
        root_certificate,
        root_key,
        client_auth=True
    )
    john_smith_client_key = create_rsa_private_key()
    john_smith_client_certificate = create_certificate(
        u"John Smith",
        john_smith_client_key,
        root_certificate,
        root_key,
        client_auth=True
    )
    jane_smith_client_key = create_rsa_private_key()
    jane_smith_client_certificate = create_certificate(
        u"Jane Smith",
        jane_smith_client_key,
        root_certificate,
        root_key,
    )

    with open("root_key.pem", "wb") as f:
        f.write(root_key.private_bytes(
            encoding=serialization.Encoding.PEM,
            format=serialization.PrivateFormat.PKCS8,
            encryption_algorithm=serialization.NoEncryption()
        ))
    with open("root_certificate.pem", "wb") as f:
        f.write(
            root_certificate.public_bytes(
                serialization.Encoding.PEM
            )
        )
    with open("server_key.pem", "wb") as f:
        f.write(server_key.private_bytes(
            encoding=serialization.Encoding.PEM,
            format=serialization.PrivateFormat.PKCS8,
            encryption_algorithm=serialization.NoEncryption()
        ))
    with open("server_certificate.pem", "wb") as f:
        f.write(
            server_certificate.public_bytes(
                serialization.Encoding.PEM
            )
        )
    with open("client_key_john_doe.pem", "wb") as f:
        f.write(john_doe_client_key.private_bytes(
            encoding=serialization.Encoding.PEM,
            format=serialization.PrivateFormat.PKCS8,
            encryption_algorithm=serialization.NoEncryption()
        ))
    with open("client_certificate_john_doe.pem", "wb") as f:
        f.write(
            john_doe_client_certificate.public_bytes(
                serialization.Encoding.PEM
            )
        )
    with open("client_key_jane_doe.pem", "wb") as f:
        f.write(jane_doe_client_key.private_bytes(
            encoding=serialization.Encoding.PEM,
            format=serialization.PrivateFormat.PKCS8,
            encryption_algorithm=serialization.NoEncryption()
        ))
    with open("client_certificate_jane_doe.pem", "wb") as f:
        f.write(
            jane_doe_client_certificate.public_bytes(
                serialization.Encoding.PEM
            )
        )
    with open("client_key_john_smith.pem", "wb") as f:
        f.write(john_smith_client_key.private_bytes(
            encoding=serialization.Encoding.PEM,
            format=serialization.PrivateFormat.PKCS8,
            encryption_algorithm=serialization.NoEncryption()
        ))
    with open("client_certificate_john_smith.pem", "wb") as f:
        f.write(
            john_smith_client_certificate.public_bytes(
                serialization.Encoding.PEM
            )
        )
    with open("client_key_jane_smith.pem", "wb") as f:
        f.write(jane_smith_client_key.private_bytes(
            encoding=serialization.Encoding.PEM,
            format=serialization.PrivateFormat.PKCS8,
            encryption_algorithm=serialization.NoEncryption()
        ))
    with open("client_certificate_jane_smith.pem", "wb") as f:
        f.write(
            jane_smith_client_certificate.public_bytes(
                serialization.Encoding.PEM
            )
        )


if __name__ == '__main__':
    main()
